#include <stdio.h>
#include "system.h"
#include "string.h"
#include "altera_avalon_pio_regs.h"
#include <unistd.h>
#include "io.h"
#include "altera_avalon_jtag_uart_regs.h"


#define NUMBER_OF_MS_PER_HOUR 3600000
#define NUMBER_OF_MS_PER_MINUTE 60000
#define NUMBER_OF_MS_PER_SECOND 1000
#define NUMBER_OF_LINKS 12
#define MULTIPLIER 65536 // 2^16

#define TOP 0
#define BOTTOM 1

#define DEBUG 0

#define VOD_15 15
#define VOD_20 20
#define VOD_25 25
#define VOD_28 28
#define VOD_29 29
#define VOD_30 30

#define PRE_TAP2_7N 0x17
#define PRE_TAP2_6N 0x16
#define PRE_TAP2_5N 0x15
#define PRE_TAP2_4N 0x14
#define PRE_TAP2_3N 0x13
#define PRE_TAP2_2N 0x12
#define PRE_TAP2_1N 0x11
#define PRE_TAP2_0  0x00 
#define PRE_TAP2_1  0x1
#define PRE_TAP2_2  0x2
#define PRE_TAP2_3  0x3
#define PRE_TAP2_4  0x4
#define PRE_TAP2_5  0x5
#define PRE_TAP2_6  0x6
#define PRE_TAP2_7  0x7

#define PRE_TAP1_16N 0x30
#define PRE_TAP1_15N 0x2F
#define PRE_TAP1_14N 0x2E
#define PRE_TAP1_13N 0x2D
#define PRE_TAP1_12N 0x2C
#define PRE_TAP1_11N 0x2B
#define PRE_TAP1_10N 0x2A
#define PRE_TAP1_9N 0x29
#define PRE_TAP1_8N 0x28
#define PRE_TAP1_7N 0x27
#define PRE_TAP1_6N 0x26
#define PRE_TAP1_5N 0x25
#define PRE_TAP1_4N 0x24
#define PRE_TAP1_3N 0x23
#define PRE_TAP1_2N 0x22
#define PRE_TAP1_1N 0x21
#define PRE_TAP1_0  0x00 
#define PRE_TAP1_1  0x1
#define PRE_TAP1_2  0x2
#define PRE_TAP1_3  0x3
#define PRE_TAP1_4  0x4
#define PRE_TAP1_5  0x5
#define PRE_TAP1_6  0x6
#define PRE_TAP1_7  0x7
#define PRE_TAP1_8  0x8
#define PRE_TAP1_9  0x9
#define PRE_TAP1_10  0xA
#define PRE_TAP1_11  0xB
#define PRE_TAP1_12  0xC
#define PRE_TAP1_13  0xD
#define PRE_TAP1_14  0xE
#define PRE_TAP1_15  0xF
#define PRE_TAP1_16  0x10

#define POST_TAP1_25N 0x59
#define POST_TAP1_24N 0x58
#define POST_TAP1_23N 0x57
#define POST_TAP1_22N 0x56
#define POST_TAP1_21N 0x55
#define POST_TAP1_20N 0x54
#define POST_TAP1_19N 0x53
#define POST_TAP1_18N 0x52
#define POST_TAP1_17N 0x51
#define POST_TAP1_16N 0x50
#define POST_TAP1_15N 0x4F
#define POST_TAP1_14N 0x4E
#define POST_TAP1_13N 0x4D
#define POST_TAP1_12N 0x4C
#define POST_TAP1_11N 0x4B
#define POST_TAP1_10N 0x4A
#define POST_TAP1_9N 0x49
#define POST_TAP1_8N 0x48
#define POST_TAP1_7N 0x47
#define POST_TAP1_6N 0x46
#define POST_TAP1_5N 0x45
#define POST_TAP1_4N 0x44
#define POST_TAP1_3N 0x43
#define POST_TAP1_2N 0x42
#define POST_TAP1_1N 0x41
#define POST_TAP1_0 0
#define POST_TAP1_1 1
#define POST_TAP1_2 2
#define POST_TAP1_3 3
#define POST_TAP1_4 4
#define POST_TAP1_5 5
#define POST_TAP1_6 6
#define POST_TAP1_7 7
#define POST_TAP1_8 8
#define POST_TAP1_9 9
#define POST_TAP1_10 10
#define POST_TAP1_11 11
#define POST_TAP1_12 12
#define POST_TAP1_13 13
#define POST_TAP1_14 14
#define POST_TAP1_15 15
#define POST_TAP1_16 16
#define POST_TAP1_17 17
#define POST_TAP1_18 18
#define POST_TAP1_19 19
#define POST_TAP1_20 20
#define POST_TAP1_21 21
#define POST_TAP1_22 22
#define POST_TAP1_23 23
#define POST_TAP1_24 24
#define POST_TAP1_25 25


#define POST_TAP2_12N 0x2C
#define POST_TAP2_11N 0x2B
#define POST_TAP2_10N 0x2A
#define POST_TAP2_9N 0x29
#define POST_TAP2_8N 0x28
#define POST_TAP2_7N 0x27
#define POST_TAP2_6N 0x26
#define POST_TAP2_5N 0x25
#define POST_TAP2_4N 0x24
#define POST_TAP2_3N 0x23
#define POST_TAP2_2N 0x22
#define POST_TAP2_1N 0x21
#define POST_TAP2_0  0x00 
#define POST_TAP2_1  0x1
#define POST_TAP2_2  0x2
#define POST_TAP2_3  0x3
#define POST_TAP2_4  0x4
#define POST_TAP2_5  0x5
#define POST_TAP2_6  0x6
#define POST_TAP2_7  0x7
#define POST_TAP2_8  0x8
#define POST_TAP2_9  0x9
#define POST_TAP2_10  0xA
#define POST_TAP2_11  0xB
#define POST_TAP2_12  0xC

#define EQ_CTRL_0 0
#define EQ_CTRL_1 1
#define EQ_CTRL_2 2
#define EQ_CTRL_3 3
#define EQ_CTRL_4 4
#define EQ_CTRL_5 5
#define EQ_CTRL_6 6
#define EQ_CTRL_7 7
#define EQ_CTRL_8 8
#define EQ_CTRL_9 9
#define EQ_CTRL_10 10
#define EQ_CTRL_11 11
#define EQ_CTRL_12 12
#define EQ_CTRL_13 13
#define EQ_CTRL_14 14
#define EQ_CTRL_15 15
#define EQ_CTRL_16 16
#define EQ_CTRL_17 17
#define EQ_CTRL_18 18
#define EQ_CTRL_19 19
#define EQ_CTRL_20 20
#define EQ_CTRL_21 21
#define EQ_CTRL_22 22
#define EQ_CTRL_23 23
#define EQ_CTRL_24 24
#define EQ_CTRL_25 25
#define EQ_CTRL_26 26
#define EQ_CTRL_27 27
#define EQ_CTRL_28 28
#define EQ_CTRL_29 29
#define EQ_CTRL_30 30
#define EQ_CTRL_31 31

//11C[3:0],11A[7:0]

#define EQ_GAIN_0 0x0
#define EQ_GAIN_1 0xE00
#define EQ_GAIN_2 0xFC0
#define EQ_GAIN_3 0xFF8
#define EQ_GAIN_4 0xFFF

#define PRBS_7 0
#define PRBS_23 1
#define PRBS_31 2
#define PRBS_15 3
#define HIGH_FREQUENCY 4
#define LOW_FREQUENCY 5
#define PRBS_9 6


/* Functions */

/*-----------------------------------------------
 * input_char()
 * 
 * Gets a single input char for menu prompt
 * 
 *   ******** Added by Guy for Menu inputs
 *   ******** Changed by Maurizio for Nios II SDK Shell support
 * 
 * ----------------------------------------------*/
char input_char(void)
 {
 char in_char, rx_char;
 
   rx_char = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
         rx_char = in_char;
     else if (in_char == '\n')
         break;
   };
 return (rx_char);
 }

int input_number(void)
 {
 char in_char,LSB;
 int temp, number;
 
   LSB = 0;
   temp = 0;
   number = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;        
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
         
    
          number  = temp;         
   };
 return (number);
 } 
 
int input_byte(void)
 {
 char in_char, MSB,LSB;
 int temp, High_nibble, Low_nibble, Byte;
 
   MSB = 0;
   LSB = 0;
   temp = 0;
   Byte = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         MSB = LSB;         
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (MSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          High_nibble = temp;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          Low_nibble = temp;  
    
          Byte = (High_nibble << 4) | Low_nibble;         
   };
 return (Byte);
 } 


int input_double(void)
 {
 char in_char, MSB,LSB;
 int temp, High_nibble, Low_nibble, Double_Digit;
 
   MSB = 0;
   LSB = 0;
   temp = 0;
   Double_Digit = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         MSB = LSB;         
         LSB = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (MSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            default : break;
            }
          
          High_nibble = temp;
          
            switch (LSB)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            default : break;
            }
          
          Low_nibble = temp;  
    
          Double_Digit = (High_nibble *10) + Low_nibble;         
   };
 return (Double_Digit);
 } 
 
 
int input_word(void)
 {
 char in_char, N3,N2,N1,N0;
 int temp, N3_int, N2_int,N1_int,N0_int, Word;
 
   N3 = 0;
   N2 = 0;
   N1 = 0;
   N0 = 0;         
   temp = 0;
   Word = 0;
   while (1)
   {
     in_char = getchar();
     if (in_char != '\r' && in_char != '\n')
        {
         //rx_char = in_char;
         N3 = N2;
         N2 = N1;
         N1 = N0;
         N0 = in_char;
        }
     else if (in_char == '\n')
         break;
     switch (N3)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N3_int = temp;
          
         switch (N2)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N2_int = temp;  

         switch (N1)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N1_int = temp;  

         switch (N0)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          
          N0_int = temp;                
          Word = (N3_int << 12) | (N2_int << 8) | (N1_int << 4) | N0_int;         
   };
 return (Word);
 } 
 



int Read_Channel_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL0_REG_BASE);         		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL1_REG_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL2_REG_BASE);   
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL3_REG_BASE);               
        break;   
	  case 4 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL4_REG_BASE);         		   
        break;     
      case 5:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL5_REG_BASE);   
        break;       
      case 6:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL6_REG_BASE);   
        break;         
      case 7:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL7_REG_BASE);               
        break;  
	  case 8 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL8_REG_BASE);         		   
        break;     
      case 9:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL9_REG_BASE);   
        break;       
      case 10:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL10_REG_BASE);   
        break;         
      case 11:
        temp = IORD_ALTERA_AVALON_PIO_DATA(CHANNEL11_REG_BASE);               
        break;  		
      default: 
			
		temp = 0;
      }    
 return (temp);      
}

void rmw_channel (int offset,int bitmask, int newval)
 {

	 int value;

	  //Read data from data register at offset
	  
	  value = IORD(RECONFIG_MGMT_0_BASE,offset);

	   // bitwise-AND and clear out bitmask bits
	 
	  value = value & (0xff & (~bitmask));
	
	  value = 0x000000ff & value; // keep only lower 8 bits.
	 
	  value = value | newval;
	  
	  IOWR(RECONFIG_MGMT_0_BASE,offset,value);	  
} 

int rd_channel (int offset)
 {

	 int value;
	  
	  value = IORD(RECONFIG_MGMT_0_BASE,offset); 
	  
	 return(value);
} 

void rmw_lc (int offset,int bitmask, int newval)
 {

	 int value;

	  //Read data from data register at offset
	  
	  value = IORD(RECONFIG_MGMT_LC_BASE,offset);

	   // bitwise-AND and clear out bitmask bits
	 
	  value = value & (0xff & (~bitmask));
	
	  value = 0x000000ff & value; // keep only lower 8 bits.
	 
	  value = value | newval;
	  
	  IOWR(RECONFIG_MGMT_LC_BASE,offset,value);	  
} 

int rd_lc (int offset)
 {

	 int value;
	  
	  value = IORD(RECONFIG_MGMT_LC_BASE,offset); 
	  
	 return(value);
} 

void set_default_setting(void)
{
	int j;
	
     //Make sure serialization factor is correctly set (could be changed because of TTK)

  		for (j = 0; j < NUMBER_OF_LINKS; j++)
		{	
		
		//Set upper 4 bits of register 0xA back to 0x0
		rmw_channel( ((j<<10) + 0xA),0xF0,0x0);

		//Set upper 4 bits of register 0xB back to 0x0
		rmw_channel( ((j<<10) + 0xB),0xF0,0x0);

		//Set register 0x13F back to 0x87 this sets the serialization factor back to 40.
		rmw_channel( ((j<<10) + 0x13F),0xFF,0x87);
	
		}
	}
int Read_ErrorCount_L_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_0_BASE);          		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_1_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_2_BASE);    
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_3_BASE);        
        break;  
	   case 4 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_4_BASE);          		   
        break;     
      case 5:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_5_BASE);   
        break;       
      case 6:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_6_BASE);    
        break;         
      case 7:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_7_BASE);        
        break;
	   case 8 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_8_BASE);          		   
        break;     
      case 9:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_9_BASE);   
        break;       
      case 10:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_10_BASE);    
        break;         
      case 11:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_L_11_BASE);        
        break;		
      default: 
		temp = 0;
      }    
 return (temp);      
}

int Read_ErrorCount_H_Reg( int SelectedChannel)
 {  
int temp;
	switch(SelectedChannel)
	  {
	  case 0 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_0_BASE);          		   
        break;     
      case 1:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_1_BASE);   
        break;       
      case 2:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_2_BASE);    
        break;         
      case 3:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_3_BASE);        
        break;  
	   case 4 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_4_BASE);          		   
        break;     
      case 5:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_5_BASE);   
        break;       
      case 6:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_6_BASE);    
        break;         
      case 7:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_7_BASE);        
        break;
	   case 8 :       
		  temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_8_BASE);          		   
        break;     
      case 9:           		
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_9_BASE);   
        break;       
      case 10:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_10_BASE);    
        break;         
      case 11:
        temp = IORD_ALTERA_AVALON_PIO_DATA(ERRORCOUNT_H_11_BASE);        
        break;		
      default: 
		temp = 0;
      }    
 return (temp);      
}





/* Functions */

int main()
{ 
	char rx_char;
  int Control_Reg;
  int Control2_Reg;
  int Bitrate;
  float Bitrate_temp;
  int Refclockmultiplier;
  
  int RefClock_Reg;
  int Channel_Reg[NUMBER_OF_LINKS];
  int Counter_1ms_Reg;
  float BER[NUMBER_OF_LINKS];
  float Totalbits;
  int Hours;
  int Minutes;
  int Seconds;
  int temp;
  int PrbsSelect[NUMBER_OF_LINKS];
  int PrbsPattern[NUMBER_OF_LINKS];

  int Locked[NUMBER_OF_LINKS];
  int Powerdown[NUMBER_OF_LINKS];
  int PLL_Locked[NUMBER_OF_LINKS];
  int Rx_FreqLocked[NUMBER_OF_LINKS];
  float ErrorCount[NUMBER_OF_LINKS]; 
  int ErrorCount_Reg_L[NUMBER_OF_LINKS];
  int ErrorCount_Reg_H[NUMBER_OF_LINKS];
//  int BitCount_L[NUMBER_OF_LINKS];
//  int BitCount_H[NUMBER_OF_LINKS];  
  int Loopback[NUMBER_OF_LINKS];
  int tx_vodctrl[NUMBER_OF_LINKS];
  int tx_pretap_1[NUMBER_OF_LINKS];
  int tx_pretap_2[NUMBER_OF_LINKS];
  int tx_posttap_1[NUMBER_OF_LINKS];
  int tx_posttap_2[NUMBER_OF_LINKS];  
  int rx_eqdcgain[NUMBER_OF_LINKS];
  int rx_eqctrl[NUMBER_OF_LINKS];  
  int SelectedChannel;
  int i;
  int j;
  int Show_ErrorCount;
  int BERInterval;
//  int EyeQInterval;
  int TimeInterval;
  int Reg_data;
  int Ravail;
  int High_nibble;
  int Low_nibble;
  int Rev_Serial[NUMBER_OF_LINKS];
  int Rev_Parallel_Available[NUMBER_OF_LINKS];
  int Rev_Parallel[NUMBER_OF_LINKS];  


  
//    float eye;
//  int eye_phase;
//  float BER_Minimum;
//  int BER_Minimum_phase;
//  int start_value;
//  int end_value;
//  int optimum_phase;
//  int table_phase_start;
//  int index;  
//  
//  int eye1d;
//  int eyeqbw;
//  int eyeq_status;
//  int eyeq_control;
//  int eyepolarity;
//  
//  float ErrorCount_Array[32][64]; 
//  float BER_Array[32][64];
//  
//  float ErrorCount_Array_Bottom[32][64]; 
//  float BER_Array_Bottom[32][64];
  
//  int veye;
//  int veye_top;
//  int veye_bottom;
  
//  int tap1;
//  int tap2;
//  int tap3;
//  int tap4;
//  int tap5;  
  //char line[100]; 
//  int x,y;
//  int Temp_Measurement ;
//  int pol2;
//  int pol3;
//  int pol4;
//  int pol5;
//  int tap1_histogram[16];
  //int value[16];
  int dump_register_before[600];
  int dump_register_after[600];


	 
   /**********************************************************************/
  /*  Overwrite fix and dyn values of CDR                                      */
  /**********************************************************************/ 

//		for (j = 0; j < NUMBER_OF_LINKS; j++)
//		{	
//		
//		//Set dyn to 31 : 0x1F on the position of 0x7C becomes 0x7C;
//		rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
//		
//		//Set fix to 3 (0x3).
//		rmw_channel( ((j<<10) + 0x136),0x0F,0x03);
//		rmw_channel( ((j<<10) + 0x134),0x40,0x00);
//		
//		//set IPD to 3 (0x18)
//		rmw_channel( ((j<<10) + 0x139),0x38,0x18);	
//		}

		
  /**********************************************************************/
  /*  Initialize all PMA Settings                                      */
  /**********************************************************************/
		
  for (j = 0; j < NUMBER_OF_LINKS ; j++)
  { 
	 tx_vodctrl[j]    = VOD_29; 
	 tx_pretap_2[j]   = PRE_TAP2_0; 
    tx_pretap_1[j]   = PRE_TAP1_0;
    tx_posttap_1[j]  = POST_TAP1_0; 
    tx_posttap_2[j]  = POST_TAP2_0;
	 rx_eqdcgain[j]   = EQ_GAIN_0;
    rx_eqctrl[j]     = EQ_CTRL_3;
	 

	 //Set VOD 0x109 bit [4:0]
	 rmw_channel( ((j<<10) + 0x109),0x1F,tx_vodctrl[j]);
	  
	 //Set PRETAP2 0x108 bit [4] and [2:0]
	 rmw_channel( ((j<<10) + 0x108),0x17,tx_pretap_2[j]);	 
	  
	 //Set PRETAP1 0x107 bit [5:0]
	 rmw_channel( ((j<<10) + 0x107),0x3F,tx_pretap_1[j]);	 	
	  
	 //Set POSTTAP1 0x105 bit [6:0]
	 rmw_channel( ((j<<10) + 0x105),0x7F,tx_posttap_1[j]);	
	  
	 //Set POSTTAP2 0x106 bit [5] and [3:0]
	 rmw_channel( ((j<<10) + 0x106),0x2F,tx_posttap_2[j]);	 
	 
	 //Set AC Gain 0x167 bit [5:1]
	 rmw_channel( ((j<<10) + 0x167),0x3E, rx_eqctrl[j] << 1);
	  
	 //Set DC Gain 0x11C bit [3:0] 0x11A bit [7:0]
	 rmw_channel( ((j<<10) + 0x11C),0x0F,rx_eqdcgain[j] >> 8);		
    rmw_channel( ((j<<10) + 0x11A),0xFF,rx_eqdcgain[j]);		  

  } 
  Control_Reg = 0x0000;      //Disable Serial loop by default
  Control2_Reg = 0x0000;

  /**********************************************************************/
  /*  Set PRBS-Pattern to PRBS31                                        */
  /**********************************************************************/  
  
    
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
	  PrbsSelect[i] = PRBS_31; 
  }

     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     Control2_Reg = (Control2_Reg & (0xFFE0)) | ( i ); // Select channel
     Control2_Reg = Control2_Reg | (0x1000); /* Latch PrbsPattern Assert */
     Control2_Reg = (Control2_Reg & (0xF8FF)) | (PrbsSelect[i] << 8); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
     
     Control2_Reg = Control2_Reg & (0xEFFF); /* Latch PrbsPattern Deassert */
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
     }
     
     Control2_Reg = Control2_Reg & (0xFFFC); 
     IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
     

  

  Control2_Reg = (Control2_Reg & (0xFFE0));
  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
 
  SelectedChannel = 0; // Default Selected Channels is channel 0 
	  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  Powerdown[i] = 0;
  Rev_Parallel[i] = 0;  
  Loopback[i] = 0;  
  }
 
  set_default_setting();
		

  
  /**********************************************************************/
  /*  Reset All Channels                                                  */
  /**********************************************************************/  
  

        Control_Reg = Control_Reg | (0x8000);
		  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

        usleep(10);
             
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

    
   
   
   Show_ErrorCount = 0;    
   
   BERInterval = 1; // Set to 1 seconds as default; 
   //EyeQInterval = 1; // Set to 10 ms as default.
   // Alternative to usleep 
   //do
   // {
   //  Counter_1ms_Reg = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE); // NUMBER_OF_MS_PER_SECOND;
   //} while (Counter_1ms_Reg < 1000);
   
   usleep(2000000); // Wait around 2 seconds for reference clock measurement
  
  /**********************************************************************/
  /*  Main Program Loop                                                 */
  /**********************************************************************/  
  while (1)
	{
 
   for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {   
  Channel_Reg[i]      =  Read_Channel_Reg(i);
  ErrorCount_Reg_L[i] =  Read_ErrorCount_L_Reg(i);
  ErrorCount_Reg_H[i] =  Read_ErrorCount_H_Reg(i);	  
  }

  
  //Temp_Measurement    = IORD_ALTERA_AVALON_PIO_DATA(TEMP_DATA_BASE);

  RefClock_Reg   = IORD_ALTERA_AVALON_PIO_DATA(BITRATE_REG_BASE);  
  
  Refclockmultiplier = (0x003F) & (Channel_Reg[0] >> 6);
    
  Bitrate_temp =  ((float)RefClock_Reg* Refclockmultiplier)/(1000000);
  
  Bitrate = (Bitrate_temp); // Convert the floats to integer numbers

  
  Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);

   
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  ErrorCount[i] = ((float) (ErrorCount_Reg_H[i]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[i]);
  }

  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
  Locked[i]  = 0x0001 & (Channel_Reg[i] >> 15);    
  PLL_Locked[i]  = 0x0001 & (Channel_Reg[i] >> 14);
  Rev_Serial[i]  = 0x0001 & (Channel_Reg[i] >> 13);  
  Rx_FreqLocked[i]   = 0x0001 & (Channel_Reg[i] >> 12); 
  Rev_Parallel_Available[i] = 0x0001 & (Channel_Reg[i]);   
  //if (Locked[i] == 1)
  //  ErrorCount[i] = 0x007F & (Channel_Reg[i]);
  //else
  //  ErrorCount[i] = 99;
  // 
  }
  

  Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);

 
  Hours =  Counter_1ms_Reg / NUMBER_OF_MS_PER_HOUR ;
  Minutes = (Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR))/NUMBER_OF_MS_PER_MINUTE;
  Seconds = (Counter_1ms_Reg - (Hours * NUMBER_OF_MS_PER_HOUR) - (Minutes * NUMBER_OF_MS_PER_MINUTE)) / NUMBER_OF_MS_PER_SECOND;
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
      if (ErrorCount[i] > 0)
        BER[i] = ((ErrorCount[i])) / Totalbits;
      else
        //BER[i] = 0;
        BER[i] = ((float) 3) / Totalbits; /* confidence level (CL) of 95% = -ln(1-CL) = 3*/
  } /* For Loop */  
  
  for (i = 0; i < NUMBER_OF_LINKS ; i++)
  {
    switch(PrbsSelect[i])
    {
      case PRBS_7 : PrbsPattern[i] = 7;break;
      case PRBS_23 : PrbsPattern[i] = 23;break;
      case PRBS_31 : PrbsPattern[i] = 31;break;
      case PRBS_15 : PrbsPattern[i] = 15;break;
      case PRBS_9 : PrbsPattern[i] = 9;break;
	 case HIGH_FREQUENCY:  PrbsPattern[i] = 0;break;
	 case LOW_FREQUENCY:  PrbsPattern[i] = 0;break;			
      default:break;
    }
  } /* For Loop */    



    printf("\n\n\n");
    printf("Arria 10 GX Devkit Multi PRBS Demo Design with ADME (TTK support) \n");
    printf("--------------------------------------------------\n");
    printf("| %2d lanes using FMCA at Bitrate %4d Mbps |\n",NUMBER_OF_LINKS, Bitrate);   
    printf("--------------------------------------------------\n");
    //printf("Serial LoopBack : %d\n",Loopback);
    printf("SelectedChannel: %2d\n",SelectedChannel );

     
     printf("Channel            :|");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
       printf("%2X|",i);
     }
     printf("\n");

     printf("                    |");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     printf("--|");
     }
     printf("\n");
	
     printf("PrbsPattern        :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     printf("%2d|",PrbsPattern[i]);
     }
     printf("\n");
	  
     printf("PowerDown          :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     printf("%2d|",Powerdown[i]);
     }
     printf("\n");

     printf("Serial Loop        :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
 	if (Loopback[i] == 1)
       	printf("%2d|",Loopback[i]);
 	else
 		printf(" 0|");
     }
     printf("\n");
	  
     printf("Rev. Serial Loop   :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
 	if (Rev_Serial[i] == 1)
       	printf("%2d|",Rev_Serial[i]);
 	else
 		printf("  |");
     }
     printf("\n");
     
     printf("Rev. Parallel Loop :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     if (Rev_Parallel_Available[i] == 1)
 	  printf("%2d|",Rev_Parallel[i]);
     else
 	  printf("  |");
     }
     printf("\n");
     
        
     printf("PLLLocked          :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0) 
       printf("%2d|",PLL_Locked[i]);
     else
       printf("  |");    
     }
     printf("\n"); 
     
     printf("FreqLocked         :|");        
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0)       
       printf("%2d|",Rx_FreqLocked[i]);
     else
       printf("  |");    
     }
     printf("\n");    
     
     printf("PrbsLocked         :|");

     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     {
     if (Powerdown[i] == 0)      
       printf("%2d|",Locked[i]);
     else
       printf("  |");
     }
     printf("\n");
   
       
     printf("Errorcount         :|");
     for (i = 0; i < NUMBER_OF_LINKS ; i++)
     { 
     if ((Locked[i] == 0) || (Powerdown[i] == 1))
       printf("  |");
     else if (ErrorCount[i] > 99)
       printf("xx|");
     else	
       printf("%2d|",ErrorCount_Reg_L[i]);      
     }
     printf("\n\n");
    
    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Powerdown[i] == 1)
        printf("Ch%2d Powered down\n",i);        
      else if (Locked[i] == 1)
      {
 	    if (ErrorCount[i] > 0)
              printf("BER Estimate  Ch %1X  : %e    (# Errors/Totalbits)\n",i,BER[i]);
          else
              printf("BER (CL=0.95) Ch %1X  : %e\n",i,BER[i]);
      }
      else
        printf("No Data Lock on Ch%2d \n",i);
    }

    for (i = 0; i < NUMBER_OF_LINKS ; i++)
    { 
      if (Show_ErrorCount  == 1)
        {
         printf("\n");
         if (Powerdown[i] == 1)
             printf("Ch%2d Powered down\n",i);        
         else if (Locked[i] == 1)    
        {
             printf("ErrorCount  Ch %1X  : %e    \n",i,ErrorCount[i]);
        }
        else
          printf("No Data Lock on Ch%2d \n",i);
        }
    }
    
    Show_ErrorCount = 0;
    
     printf("\n");
     
     if (Powerdown[0] == 0) 
        printf("Test Time             : %2dh %2dm %2ds\n",Hours,Minutes,Seconds);                
    
    printf("Reference Clock Frequency        : %7d kHz\n",RefClock_Reg/1000);
    //printf("Measured Junction Temperature Tj : %2d Celsius\n",Temp_Measurement);
       
    printf("\nSelect Action : \n");
    printf("===============\n");
    
    printf("1. Set all channels back to default and toggle Reset\n");
	 //printf("2. Force Rx digitalreset low on all Channels\n");
    printf("3. Select Channel To Control\n");
    printf("4. Show/Control Transceiver PMA Settings on Selected Channel\n");
    printf("5. Insert Biterror on Selected Channel\n");
    printf("6. Reset ErrorCounter on Selected Channel\n");
    printf("7. Show Status\n");
	 printf("8. Toggle Serial Loopback on Selected Channel \n");
    printf("A. Select Prbs Pattern on Selected Channel\n");
    printf("B. Show detailed Errorcount\n");
    printf("C. Refresh BER every %d seconds on selected channel\n",BERInterval);
    printf("D. Input new BER Time Interval\n");
    printf("E. Show Transceiver PMA Settings on all channels\n");
    //printf("F. Toggle Parallel Loopback on selected channel (if available)\n"); 	  
	 //printf("G. Input new EyeQInterval time\n");
	 //printf("M. Measure 2-D eye on selected channel using EyeQ\n");	
	 //printf("N. Measure 2-D eye on selected channel using BERB Block\n");	 
	 //printf("P. Change DCD setting on selected channel \n");
	 //printf("Q. Run DFE One-Time on selected channel\n");
	 //printf("O. Redo Offset cancellation\n");
	 //printf("R. Read Offset cancellation results\n");
	 printf("P. Redo ATX PLL calibration \n");
	 printf("S. Store all channel information in memory of Selected Channel \n");
	 printf("T. Store and compare with values stored in process S \n");
	 printf("Z. Dump channel content of selected channel \n");
	 
    printf("0. Stop Test \n\n");
    printf("  Enter Choice :");
  
    rx_char = input_char();
	 printf("%c",rx_char);
    
    
		switch (rx_char)
		{
      case '1': /*Reset settings to default and do a reset*/
		
		
		  set_default_setting();
		
        Control_Reg = Control_Reg | (0x8000);
		  IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

        usleep(10);
             
        Control_Reg = Control_Reg & (0x7FFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        
        usleep(1000000); // Wait for 1 second
        printf("All Channels set back to default and reset toggled.\n\n\n");
        break; //case '1'
		  
//     case '2': /*Force_Rx_digitalreset */ 
//
//        Control_Reg = Control_Reg | (0x0001);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//
//        usleep(10);
//       
//        
//        
//        usleep(1000000); // Wait for 1 second
//
//        break; //case '2'		  

                
      case '3': /*Select Channel To Control*/

      printf("3. Select Channel To Control\n");
      printf("Current Channel : ");
      switch (SelectedChannel)
       {
       case  0  : printf(" 0: FMCA_DP_P0 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[0],Bitrate); break;
       case  1  : printf(" 1: FMCA_DP_P1 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[1],Bitrate); break;
       case  2  : printf(" 2: FMCA_DP_P2 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[2],Bitrate); break;
       case  3  : printf(" 3: FMCA_DP_P3 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[3],Bitrate); break;
       case  4  : printf(" 4: FMCA_DP_P4 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[4],Bitrate); break;
       case  5  : printf(" 5: FMCA_DP_P5 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[5],Bitrate); break;
       case  6  : printf(" 6: FMCA_DP_P6 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[6],Bitrate); break;
       case  7  : printf(" 7: FMCA_DP_P7 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[7],Bitrate); break;
       case  8  : printf(" 8: FMCA_DP_P8 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[8],Bitrate); break;
       case  9  : printf(" 9: FMCA_DP_P9 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[9],Bitrate); break;
       case  10  : printf(" 10: FMCA_DP_P10 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[10],Bitrate); break;
       case  11  : printf(" 11: FMCA_DP_P11 PRBS-%2d Bitrate %4d Mbps \n",PrbsPattern[11],Bitrate); break;			 
       default : break;
       }  
 		 
     temp = 0xffffffff;
  
     do {
      printf("New Channel (0-B)         :");
      rx_char = input_char(); 
  
      switch (rx_char)
      {
      case '0': temp = 0; break;
      case '1': temp = 1; break;
      case '2': temp = 2; break;
      case '3': temp = 3; break;
      case '4': temp = 4; break;
      case '5': temp = 5; break;
      case '6': temp = 6; break;
      case '7': temp = 7; break;
      case '8': temp = 8; break;
      case '9': temp = 9; break;	
		case 'a':
		case 'A': temp = 10;break;
		case 'b':
		case 'B': temp = 11;break;			
      default : break;
      }
  
    } while (temp == 0xffffffff);
    
    SelectedChannel = temp;
    
    
          /*printf("temp %d : \n",temp);
          printf("SelectedChannel %d : \n",SelectedChannel);*/
          Control2_Reg = (Control2_Reg & (0xFFE0)) | (SelectedChannel);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);      
                   

        
      break; //case '3'
                
			case '4': /*Show/Control Transceiver Parameters */
				
//Read settings first (as they can have been set differnetly in e.g. Transceiver toolkit or through scripts

	 tx_vodctrl[SelectedChannel]    = rd_channel((SelectedChannel << 10) + 0x109) & 0x1F; 
	 tx_pretap_2[SelectedChannel]   = rd_channel((SelectedChannel << 10) + 0x108) & 0x17; 
    tx_pretap_1[SelectedChannel]   = rd_channel((SelectedChannel << 10) + 0x107) & 0x3F; 
    tx_posttap_1[SelectedChannel]  = rd_channel((SelectedChannel << 10) + 0x105) & 0x7F; 
    tx_posttap_2[SelectedChannel]  = rd_channel((SelectedChannel << 10) + 0x106) & 0x2F; 	 
	 rx_eqctrl[SelectedChannel]     = (rd_channel((SelectedChannel << 10) + 0x167) & 0x3E) >> 1;
	 rx_eqdcgain[SelectedChannel]   = ((rd_channel((SelectedChannel << 10) + 0x11C) & 0x0F) << 8) + (rd_channel((SelectedChannel << 10) + 0x11A) & 0xFF);
	   
		  

		  printf("\n");
        printf("4. Transceiver PMA Settings of Channel %1d\n",SelectedChannel);
		  printf("    VOD Level             : %2d\n", tx_vodctrl[SelectedChannel]);
 
           switch (tx_pretap_2[SelectedChannel])
           {            
           case  PRE_TAP2_7N  : printf("    Pre Tap 2 Level       : -7\n"); break;
           case  PRE_TAP2_6N  : printf("    Pre Tap 2 Level       : -6\n"); break;
           case  PRE_TAP2_5N  : printf("    Pre Tap 2 Level       : -5\n"); break;
           case  PRE_TAP2_4N  : printf("    Pre Tap 2 Level       : -4\n"); break;
           case  PRE_TAP2_3N  : printf("    Pre Tap 2 Level       : -3\n"); break;
           case  PRE_TAP2_2N  : printf("    Pre Tap 2 Level       : -2\n"); break;
           case  PRE_TAP2_1N  : printf("    Pre Tap 2 Level       : -1\n"); break;
           case  PRE_TAP2_0   : printf("    Pre Tap 2 Level       :  0\n"); break;
           case  PRE_TAP2_1   : printf("    Pre Tap 2 Level       :  1\n"); break;
           case  PRE_TAP2_2   : printf("    Pre Tap 2 Level       :  2\n"); break;
           case  PRE_TAP2_3   : printf("    Pre Tap 2 Level       :  3\n"); break;
           case  PRE_TAP2_4   : printf("    Pre Tap 2 Level       :  4\n"); break;
           case  PRE_TAP2_5   : printf("    Pre Tap 2 Level       :  5\n"); break;
           case  PRE_TAP2_6   : printf("    Pre Tap 2 Level       :  6\n"); break;
           case  PRE_TAP2_7   : printf("    Pre Tap 2 Level       :  7\n"); break;           
           default : break;
           }			  
           
           switch (tx_pretap_1[SelectedChannel])
           {
           case  PRE_TAP1_16N : printf("    Pre Tap 1 Level       : -16\n"); break;				 
           case  PRE_TAP1_15N : printf("    Pre Tap 1 Level       : -15\n"); break;
           case  PRE_TAP1_14N : printf("    Pre Tap 1 Level       : -14\n"); break;
           case  PRE_TAP1_13N : printf("    Pre Tap 1 Level       : -13\n"); break;
           case  PRE_TAP1_12N : printf("    Pre Tap 1 Level       : -12\n"); break;
           case  PRE_TAP1_11N : printf("    Pre Tap 1 Level       : -11\n"); break;
           case  PRE_TAP1_10N : printf("    Pre Tap 1 Level       : -10\n"); break;
           case  PRE_TAP1_9N  : printf("    Pre Tap 1 Level       : -9\n"); break;  
           case  PRE_TAP1_8N  : printf("    Pre Tap 1 Level       : -8\n"); break;              
           case  PRE_TAP1_7N  : printf("    Pre Tap 1 Level       : -7\n"); break;
           case  PRE_TAP1_6N  : printf("    Pre Tap 1 Level       : -6\n"); break;
           case  PRE_TAP1_5N  : printf("    Pre Tap 1 Level       : -5\n"); break;
           case  PRE_TAP1_4N  : printf("    Pre Tap 1 Level       : -4\n"); break;
           case  PRE_TAP1_3N  : printf("    Pre Tap 1 Level       : -3\n"); break;
           case  PRE_TAP1_2N  : printf("    Pre Tap 1 Level       : -2\n"); break;
           case  PRE_TAP1_1N  : printf("    Pre Tap 1 Level       : -1\n"); break;
           case  PRE_TAP1_0   : printf("    Pre Tap 1 Level       :  0\n"); break;
           case  PRE_TAP1_1   : printf("    Pre Tap 1 Level       :  1\n"); break;
           case  PRE_TAP1_2   : printf("    Pre Tap 1 Level       :  2\n"); break;
           case  PRE_TAP1_3   : printf("    Pre Tap 1 Level       :  3\n"); break;
           case  PRE_TAP1_4   : printf("    Pre Tap 1 Level       :  4\n"); break;
           case  PRE_TAP1_5   : printf("    Pre Tap 1 Level       :  5\n"); break;
           case  PRE_TAP1_6   : printf("    Pre Tap 1 Level       :  6\n"); break;
           case  PRE_TAP1_7   : printf("    Pre Tap 1 Level       :  7\n"); break; 
           case  PRE_TAP1_8   : printf("    Pre Tap 1 Level       :  8\n"); break;
           case  PRE_TAP1_9   : printf("    Pre Tap 1 Level       :  9\n"); break;
           case  PRE_TAP1_10  : printf("    Pre Tap 1 Level       :  10\n"); break;
           case  PRE_TAP1_11  : printf("    Pre Tap 1 Level       :  11\n"); break;
           case  PRE_TAP1_12  : printf("    Pre Tap 1 Level       :  12\n"); break;
           case  PRE_TAP1_13  : printf("    Pre Tap 1 Level       :  13\n"); break;
           case  PRE_TAP1_14  : printf("    Pre Tap 1 Level       :  14\n"); break;
           case  PRE_TAP1_15  : printf("    Pre Tap 1 Level       :  15\n"); break;
           case  PRE_TAP1_16  : printf("    Pre Tap 1 Level       :  16\n"); break; 					  
           default : break;
           }

           switch (tx_posttap_1[SelectedChannel])
           {
           case  POST_TAP1_25N : printf("    Post Tap 1 Level         : -25\n"); break;
           case  POST_TAP1_24N : printf("    Post Tap 1 Level         : -24\n"); break;
           case  POST_TAP1_23N : printf("    Post Tap 1 Level         : -23\n"); break;
           case  POST_TAP1_22N : printf("    Post Tap 1 Level         : -22\n"); break;
           case  POST_TAP1_21N : printf("    Post Tap 1 Level         : -21\n"); break;
           case  POST_TAP1_20N : printf("    Post Tap 1 Level         : -20\n"); break;
           case  POST_TAP1_19N : printf("    Post Tap 1 Level         : -19\n"); break;  
           case  POST_TAP1_18N : printf("    Post Tap 1 Level         : -18\n"); break;              
           case  POST_TAP1_17N : printf("    Post Tap 1 Level         : -17\n"); break;
			  case  POST_TAP1_16N : printf("    Post Tap 1 Level         : -16\n"); break;				 
           case  POST_TAP1_15N : printf("    Post Tap 1 Level         : -15\n"); break;
           case  POST_TAP1_14N : printf("    Post Tap 1 Level         : -14\n"); break;
           case  POST_TAP1_13N : printf("    Post Tap 1 Level         : -13\n"); break;
           case  POST_TAP1_12N : printf("    Post Tap 1 Level         : -12\n"); break;
           case  POST_TAP1_11N : printf("    Post Tap 1 Level         : -11\n"); break;
           case  POST_TAP1_10N : printf("    Post Tap 1 Level         : -10\n"); break;
           case  POST_TAP1_9N  : printf("    Post Tap 1 Level         : -9\n"); break;  
           case  POST_TAP1_8N  : printf("    Post Tap 1 Level         : -8\n"); break;              
           case  POST_TAP1_7N  : printf("    Post Tap 1 Level         : -7\n"); break;
           case  POST_TAP1_6N  : printf("    Post Tap 1 Level         : -6\n"); break;
           case  POST_TAP1_5N  : printf("    Post Tap 1 Level         : -5\n"); break;
           case  POST_TAP1_4N  : printf("    Post Tap 1 Level         : -4\n"); break;
           case  POST_TAP1_3N  : printf("    Post Tap 1 Level         : -3\n"); break;
           case  POST_TAP1_2N  : printf("    Post Tap 1 Level         : -2\n"); break;
           case  POST_TAP1_1N  : printf("    Post Tap 1 Level         : -1\n"); break;				  
           case  POST_TAP1_0  : printf("    Post Tap 1 Level      :  0\n"); break;
           case  POST_TAP1_1  : printf("    Post Tap 1 Level      :  1\n"); break;    
           case  POST_TAP1_2  : printf("    Post Tap 1 Level      :  2\n"); break;
           case  POST_TAP1_3  : printf("    Post Tap 1 Level      :  3\n"); break;   
           case  POST_TAP1_4  : printf("    Post Tap 1 Level      :  4\n"); break;
           case  POST_TAP1_5  : printf("    Post Tap 1 Level      :  5\n"); break;   
           case  POST_TAP1_6  : printf("    Post Tap 1 Level      :  6\n"); break;
           case  POST_TAP1_7  : printf("    Post Tap 1 Level      :  7\n"); break;   
           case  POST_TAP1_8  : printf("    Post Tap 1 Level      :  8\n"); break;
           case  POST_TAP1_9  : printf("    Post Tap 1 Level      :  9\n"); break;   
           case  POST_TAP1_10 : printf("    Post Tap 1 Level      : 10\n"); break;
           case  POST_TAP1_11 : printf("    Post Tap 1 Level      : 11\n"); break;   
           case  POST_TAP1_12 : printf("    Post Tap 1 Level      : 12\n"); break;     
           case  POST_TAP1_13 : printf("    Post Tap 1 Level      : 13\n"); break;   
           case  POST_TAP1_14 : printf("    Post Tap 1 Level      : 14\n"); break;
           case  POST_TAP1_15 : printf("    Post Tap 1 Level      : 15\n"); break;   
           case  POST_TAP1_16 : printf("    Post Tap 1 Level      : 16\n"); break;
           case  POST_TAP1_17 : printf("    Post Tap 1 Level      : 17\n"); break;   
           case  POST_TAP1_18 : printf("    Post Tap 1 Level      : 18\n"); break;
           case  POST_TAP1_19 : printf("    Post Tap 1 Level      : 19\n"); break;   
           case  POST_TAP1_20 : printf("    Post Tap 1 Level      : 20\n"); break;
           case  POST_TAP1_21 : printf("    Post Tap 1 Level      : 21\n"); break;   
           case  POST_TAP1_22 : printf("    Post Tap 1 Level      : 22\n"); break; 
           case  POST_TAP1_23 : printf("    Post Tap 1 Level      : 23\n"); break;   
           case  POST_TAP1_24 : printf("    Post Tap 1 Level      : 24\n"); break;
           case  POST_TAP1_25 : printf("    Post Tap 1 Level      : 25\n"); break;             
           default : break;
           }

           switch (tx_posttap_2[SelectedChannel])
           {
           case  POST_TAP2_12N : printf("    Post Tap 2 Level      : -12\n"); break;
           case  POST_TAP2_11N : printf("    Post Tap 2 Level      : -11\n"); break;
           case  POST_TAP2_10N : printf("    Post Tap 2 Level      : -10\n"); break;
           case  POST_TAP2_9N  : printf("    Post Tap 2 Level      : -9\n"); break;  
           case  POST_TAP2_8N  : printf("    Post Tap 2 Level      : -8\n"); break;              
           case  POST_TAP2_7N  : printf("    Post Tap 2 Level      : -7\n"); break;
           case  POST_TAP2_6N  : printf("    Post Tap 2 Level      : -6\n"); break;
           case  POST_TAP2_5N  : printf("    Post Tap 2 Level      : -5\n"); break;
           case  POST_TAP2_4N  : printf("    Post Tap 2 Level      : -4\n"); break;
           case  POST_TAP2_3N  : printf("    Post Tap 2 Level      : -3\n"); break;
           case  POST_TAP2_2N  : printf("    Post Tap 2 Level      : -2\n"); break;
           case  POST_TAP2_1N  : printf("    Post Tap 2 Level      : -1\n"); break;
           case  POST_TAP2_0   : printf("    Post Tap 2 Level      :  0\n"); break;
           case  POST_TAP2_1   : printf("    Post Tap 2 Level      :  1\n"); break;
           case  POST_TAP2_2   : printf("    Post Tap 2 Level      :  2\n"); break;
           case  POST_TAP2_3   : printf("    Post Tap 2 Level      :  3\n"); break;
           case  POST_TAP2_4   : printf("    Post Tap 2 Level      :  4\n"); break;
           case  POST_TAP2_5   : printf("    Post Tap 2 Level      :  5\n"); break;
           case  POST_TAP2_6   : printf("    Post Tap 2 Level      :  6\n"); break;
           case  POST_TAP2_7   : printf("    Post Tap 2 Level      :  7\n"); break; 
           case  POST_TAP2_8   : printf("    Post Tap 2 Level      :  8\n"); break;
           case  POST_TAP2_9   : printf("    Post Tap 2 Level      :  9\n"); break;
           case  POST_TAP2_10  : printf("    Post Tap 2 Level      :  10\n"); break;
           case  POST_TAP2_11  : printf("    Post Tap 2 Level      :  11\n"); break;
           case  POST_TAP2_12  : printf("    Post Tap 2 Level      :  12\n"); break;
                                                                                                                                                                                                           
           default : break;
           }

           switch (rx_eqdcgain[SelectedChannel])
           {
           case  EQ_GAIN_0    : printf("    CTLE DC Gain          :  0 (DC Gain ~ -10dB) \n"); break;
           case  EQ_GAIN_1    : printf("    CTLE DC Gain          :  1 (DC Gain ~ -7dB)\n"); break;
           case  EQ_GAIN_2    : printf("    CTLE DC Gain          :  2 (DC Gain ~ -5dB)\n"); break;   
           case  EQ_GAIN_3    : printf("    CTLE DC Gain          :  3 (DC Gain ~ -3dB)\n"); break;
           case  EQ_GAIN_4    : printf("    CTLE DC Gain          :  4 (DC Gain ~  0dB)\n"); break;
				  
           default : break;
           }

			  printf("    CTLE AC Gain          : %2d\n",rx_eqctrl[SelectedChannel]);
			  
			  printf("\n");
			  
 
        printf("    New VOD Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide number from 12-31 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 12) || (temp > 31));
		  
		  printf("%2d",temp);
        tx_vodctrl[SelectedChannel] = temp; 
        
        printf("\n\n");  
       

        printf("    New Pre Tap 2 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-7 : "); 
		  do 
		  {
		  temp = input_number(); 
		  } while (temp > 7);		  
		  printf("%2d",temp);
        printf("\n\n"); 	
		  
        printf("    New Pre Tap 2 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide either + or -   : ");  
//		  do 
//		  {
		  rx_char = input_char(); 
//		  printf("%c",rx_char);
//		  } while ((rx_char != '+') || (rx_char != '-'));		  // condition is not accepted?
		  printf("%c",rx_char);
		  
		  if (rx_char == '-')
			  tx_pretap_2[SelectedChannel] = 0x10 + temp;
		  else
			  tx_pretap_2[SelectedChannel] = temp;       
           
        printf("\n\n");


        printf("    New Pre Tap 1 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide number from 0-12 : "); 
  		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 12)); 
		  printf("%2d",temp);
        printf("\n\n");		  
        printf("    New Pre Tap 1 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  printf("%c",rx_char);
		  
		  if (rx_char == '-')
			  tx_pretap_1[SelectedChannel] = 0x20 + temp;
		  else
			  tx_pretap_1[SelectedChannel] = temp;       
           
        printf("\n\n");
    
        
        printf("    New Post Tap 1 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide number from 0-25 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 25));
		  printf("%2d",temp);
        printf("\n\n");	
		  
        printf("    New Post Tap 1 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================              \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  printf("%c",rx_char);
		  
		  if (rx_char == '-')
			  tx_posttap_1[SelectedChannel] = 0x40 + temp;
		  else
			  tx_posttap_1[SelectedChannel] = temp;       
           
        printf("\n\n");  

 
        printf("    New Post Tap 2 Level Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide number from 0-12 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 12)); 
		  printf("%2d",temp);
        printf("\n\n");
		  
        printf("    New Post Tap 2 Polarity Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    Provide either + or -    : ");  
		  rx_char = input_char();
		  printf("%c",rx_char);
		  
		  if (rx_char == '-')
			  tx_posttap_2[SelectedChannel] = 0x20 + temp;
		  else
			  tx_posttap_2[SelectedChannel] = temp;   
		  
		   printf("\n\n"); 

        printf("    New CTLE AC Gain Channel %2d               \n",SelectedChannel);
        printf("    ==================================               \n");
        printf("    Provide number from 0-31 : "); 
		  do 
		  {
		  temp = input_double(); 
		  } while ((temp < 0) || (temp > 31));
		  printf("%2d",temp);
        printf("\n\n");
		  
		  rx_eqctrl[SelectedChannel] = temp; 
		  
		  printf("\n"); 

       do {
        printf("\n");  
        
        printf("    New Equalizer DC Gain Channel %2d               \n",SelectedChannel);
        printf("    ==================================                \n");
        printf("    For Equalizer DC Gain  =  0 choose '0' \n");
        printf("    For Equalizer DC Gain  =  1 choose '1' \n");
        printf("    For Equalizer DC Gain  =  2 choose '2' \n");     
        printf("    For Equalizer DC Gain  =  3 choose '3' \n");
        printf("    For Equalizer DC Gain  =  4 choose '4' \n");		 
        printf("    No Change                   choose 'X' \n\n");    
        printf("    Choice :");      
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
		  printf("%c",rx_char);
        switch (rx_char)
        {
        case '0': temp =  EQ_GAIN_0; break;
        case '1': temp =  EQ_GAIN_1; break;
        case '2': temp =  EQ_GAIN_2; break;
        case '3': temp =  EQ_GAIN_3; break;
        case '4': temp =  EQ_GAIN_4; break;          
        case 'x': 
        case 'X': temp =  rx_eqdcgain[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        rx_eqdcgain[SelectedChannel] = temp; 

		  
		  printf("\n"); 
        
        // Debug Purposes
         printf("    tx_vodctrl   = %d               \n",tx_vodctrl[SelectedChannel]);
         printf("    tx_pretap_2  = %x              \n",tx_pretap_1[SelectedChannel]);		  
         printf("    tx_pretap_1  = %x              \n",tx_pretap_1[SelectedChannel]);
         printf("    tx_posttap_1 = %x              \n",tx_posttap_1[SelectedChannel]);
         printf("    tx_posttap_2 = %x              \n",tx_posttap_2[SelectedChannel]);
         printf("    rx_eqdcgain  = %x              \n",rx_eqdcgain[SelectedChannel] );
         printf("    rx_eqctrl    = %d              \n",rx_eqctrl[SelectedChannel] );

			 //Set VOD 0x109 bit [4:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x109),0x1F,tx_vodctrl[SelectedChannel]);
			  
			 //Set PRETAP2 0x108 bit [4] and [2:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x108),0x17,tx_pretap_2[SelectedChannel]);	 
			  
			 //Set PRETAP1 0x107 bit [5:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x107),0x3F,tx_pretap_1[SelectedChannel]);	 	
			  
			 //Set POSTTAP1 0x105 bit [6:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x105),0x7F,tx_posttap_1[SelectedChannel]);	
			  
			 //Set POSTTAP2 0x106 bit [5] and [3:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x106),0x2F,tx_posttap_2[SelectedChannel]);	 
			 
			 //Set AC Gain 0x167 bit [5:1]
			 rmw_channel( ((SelectedChannel<<10) + 0x167),0x3E, rx_eqctrl[SelectedChannel] << 1);
			  
			 //Set DC Gain 0x11C bit [3:0] 0x11A bit [7:0]
			 rmw_channel( ((SelectedChannel<<10) + 0x11C),0x0F,rx_eqdcgain[SelectedChannel] >> 8);		
			 rmw_channel( ((SelectedChannel<<10) + 0x11A),0xFF,rx_eqdcgain[SelectedChannel]);		  		  

                      
				printf("\nTransceiver PMA parameters of Channel %1d updated\n\n",SelectedChannel);
				
				break; //case '4'
        
			case '5': /* Insert Biterrors */
     
        printf("5. Insert Biterror on Channel %1d\n\n\n",SelectedChannel);


        Control_Reg = Control_Reg | (0x4000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xBFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
   
         
		break;
	
			case '6': /* Reset ErrorCounter */
        
	      printf("6. Reset ErrorCounter on Selected Channel %1d\n\n\n",SelectedChannel);
   
        
        Control_Reg = Control_Reg | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
       
        usleep(1000000);     
            
       break;
	
      case '7': /* Show Status */ 
            
       break;      

      case '8': /*Toggle Serial Loopback on selected Channel and reset */
         

        	if (Loopback[SelectedChannel] == 0)
        	       Loopback[SelectedChannel] = 1;
        	else
        	       Loopback[SelectedChannel] = 0; 
      
      
          Control_Reg = (Control_Reg & (0xFFFE)) | (Loopback[0]);
          Control_Reg = (Control_Reg & (0xFFFD)) | (Loopback[1] << 1);
          Control_Reg = (Control_Reg & (0xFFFB)) | (Loopback[2] << 2);
          Control_Reg = (Control_Reg & (0xFFF7)) | (Loopback[3] << 3); 
//			 Control_Reg = (Control_Reg & (0xFFEF)) | (Loopback[4] << 4);
//			 Control_Reg = (Control_Reg & (0xFFDF)) | (Loopback[5] << 5);   
//			 Control_Reg = (Control_Reg & (0xFFBF)) | (Loopback[6] << 6);
//			 Control_Reg = (Control_Reg & (0xFF7F)) | (Loopback[7] << 7); 
//			 Control_Reg = (Control_Reg & (0xFEFF)) | (Loopback[8] << 8);
//			 Control_Reg = (Control_Reg & (0xFDFF)) | (Loopback[9] << 9);	
//	       Control_Reg = (Control_Reg & (0xFBFF)) | (Loopback[10] << 10);
//          Control_Reg = (Control_Reg & (0xF7FF)) | (Loopback[11] << 11); 
			
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
          
          Control_Reg = Control_Reg | (0x8000);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
          usleep(10);
          
          Control_Reg = Control_Reg & (0x7FFF);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
          
						 
          
		  usleep(1000000); 
			
        Control_Reg = Control_Reg | (0x2000);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
        
        Control_Reg = Control_Reg & (0xDFFF);
        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
       
          
		  
        usleep(1000000);
        
        printf("Toggled Serial Loopback on selected channel and reset .\n\n");
        break;
     

      
      case 'a':
      case 'A': /*Select Prbs Pattern*/

      printf("A. Select Prsb Pattern on Selected Channel %1d and Reset\n",SelectedChannel);
      printf("    Current Prbs Pattern Selected   : ");
      switch (PrbsSelect[SelectedChannel])
       {
       case  0  : printf(" PRBS-7\n"); break;
       case  1  : printf(" PRBS-23\n"); break;
       case  2  : printf(" PRBS-31\n"); break;
       case  3  : printf(" PRBS-15\n"); break;
       case  4  : printf(" High Frequency Pattern (no verification)\n"); break;
		 case  5  : printf(" Low Frequency Pattern (no verification)\n"); break;		
       default : break;
       }  
     
     
        do {
        printf("\n");  
        
        printf("    New Prbs Pattern on Channel %1d               \n",SelectedChannel);
        printf("    ============================                \n");
        printf("    For PRBS-7   choose '0' \n");
        printf("    For PRBS-9   choose '1' \n");
        printf("    For PRBS-15  choose '2' \n"); 
        printf("    For PRBS-23  choose '3' \n");
        printf("    For PRBS-31  choose '4' \n");
        printf("    For High Frequency Pattern choose '5' \n");  
        printf("    For Low Frequency Pattern choose '6' \n"); 			  
        printf("    No Change    choose 'X' \n\n");    
        printf("    Choice :");             
        
        temp = 0xffffffff;
                                             
        rx_char   = input_char();
        switch (rx_char)
        {
        case '0': temp =  PRBS_7; break;
        case '1': temp =  PRBS_9; break;
        case '2': temp =  PRBS_15; break;
        case '3': temp =  PRBS_23; break; 
        case '4': temp =  PRBS_31; break; 
        case '5': temp =  HIGH_FREQUENCY; break; 
        case '6': temp =  LOW_FREQUENCY; break; 			  
        case 'x': 
        case 'X': temp =  PrbsSelect[SelectedChannel];break;
        default : break;
        }
        } while (temp == 0xffffffff);
 
        PrbsSelect[SelectedChannel] = temp; 
       
          Control2_Reg = Control2_Reg | (0x1000); /* Latch PrbsPattern Assert */
          Control2_Reg = (Control2_Reg & (0xF8FF)) | (temp << 8); 
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
          
          Control2_Reg = Control2_Reg & (0xEFFF); /* Latch PrbsPattern Deassert */
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL2_REG_BASE,Control2_Reg);
          
          Control_Reg = Control_Reg | (0x8000);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
          usleep(10);
          
          Control_Reg = Control_Reg & (0x7FFF);
          IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
          
      usleep(1000000); 
      break; //case 'A'
      
     case 'b':
     case 'B': 
        Show_ErrorCount = 1;
        break;
     
     case 'c': //continuously update BER and errorcount every BERInterval seconds
     case 'C': 

        printf("\n");  
        printf("Ch %1d BER + ErrorCount every %d seconds              \n",SelectedChannel,BERInterval);
        printf("Press any key + Enter to stop the loop\n");
        printf("=========================================               \n");
        
        temp = 0;
        
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        // Shift rigth of 16 to get access to the number of characters remaining in the read fifo
        Ravail = Reg_data >> 16;
        
        //printf("\nRavail %d :\n",Ravail);
        
        do
          {
        
        // Reset ErrorCount on Selected Channel

            Control_Reg = Control_Reg | (0x2000);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
            
            Control_Reg = Control_Reg & (0xDFFF);
            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);

            
        TimeInterval = BERInterval * 1000;
        

			do
			{
			Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
			} while (Counter_1ms_Reg < TimeInterval);
   

                     
             
    
        
            //Read ErrorCount registers
            
            Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
          
            
                     
   
           for (i = 0; i < NUMBER_OF_LINKS ; i++)
              {
				    ErrorCount_Reg_L[i] =  Read_ErrorCount_L_Reg(i);
					 ErrorCount_Reg_H[i] =  Read_ErrorCount_H_Reg(i);
              ErrorCount[i] = ((float) (ErrorCount_Reg_H[i]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[i]);
              }

            Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);

  
            for (i = 0; i < NUMBER_OF_LINKS ; i++)
            {
                if (ErrorCount[i] > 0)
                  BER[i] = ((ErrorCount[i])) / Totalbits;
                else
                  BER[i] = 0; 
            } /* For Loop */   
            

            if (Powerdown[SelectedChannel] == 1)
              printf("Ch%2d Powered down\n",SelectedChannel);        
            else if (Locked[SelectedChannel] == 1)
                {
                if (ErrorCount[SelectedChannel] > 0)
                        printf("Ch %1d : BER Estimate : %e    ErrorCount  : %e \n",SelectedChannel,BER[SelectedChannel],ErrorCount[SelectedChannel]);
                    else
                        printf("Ch %1d : BER          : 0\n",SelectedChannel);
                }
            else
              printf("No Data Lock on Ch%2d \n",SelectedChannel);
         
                
           
        temp = temp + 1;
                                           
 
        //} while (temp <= 1000); // loop a 1000 times
        Reg_data = IORD_ALTERA_AVALON_JTAG_UART_DATA(ALT_STDIN_BASE);
        Ravail = Reg_data >> 16;
        } while(!Ravail);
        rx_char   = input_char();
        
        printf("\n Stopped\n");
        
        break;

      case 'd':
      case 'D': /* New BER Time Interval */
        
        printf("D. Provide BER Time Interval\n");
        printf("   Insert a Hexadecimal Number from 01 to FF\n");
        printf("   Current BER Time Interval   : %2d seconds\n", BERInterval);
       
        temp = 0xffffffff;
           do {
            printf("   New Interval (Upper Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          High_nibble = temp;
          
          temp = 0xffffffff;
           do {
            printf("\n   New Interval (Lower Nibble) (0-F)        :");
            rx_char   = input_char();
        
            switch (rx_char)
            {
            case '0': temp = 0; break;
            case '1': temp = 1; break;
            case '2': temp = 2; break;
            case '3': temp = 3; break;
            case '4': temp = 4; break;
            case '5': temp = 5; break;
            case '6': temp = 6; break;
            case '7': temp = 7; break;
            case '8': temp = 8; break;
            case '9': temp = 9; break;
            case 'A': 
            case 'a': temp = 0xa; break;
            case 'B': 
            case 'b': temp = 0xb; break;
            case 'C': 
            case 'c': temp = 0xc; break;
            case 'D': 
            case 'd': temp = 0xd; break;
            case 'E': 
            case 'e': temp = 0xe; break;
            case 'F': 
            case 'f': temp = 0xf; break;
            default : break;
            }
          } while (temp == 0xffffffff);
          
          Low_nibble = temp;  
    
          BERInterval = (High_nibble << 4) | Low_nibble;
         
            
          printf("    New Time interval is %2d seconds:\n",BERInterval);               
            
       break; /* Case D */
       
      case 'e': /* Show Transceiver PMA Settings on all channels */
      case 'E':
			
//Read settings first (as they can have been set differnetly in e.g. Transceiver toolkit or through scripts
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
				 tx_vodctrl[i]    = rd_channel((i << 10) + 0x109) & 0x1F; 
				 tx_pretap_2[i]   = rd_channel((i << 10) + 0x108) & 0x17; 
				 tx_pretap_1[i]   = rd_channel((i << 10) + 0x107) & 0x3F; 
				 tx_posttap_1[i]  = rd_channel((i << 10) + 0x105) & 0x7F; 
				 tx_posttap_2[i]  = rd_channel((i << 10) + 0x106) & 0x2F; 	 
				 rx_eqctrl[i]     = (rd_channel((i << 10) + 0x167) & 0x3E) >> 1;
				 rx_eqdcgain[i]   = ((rd_channel((i << 10) + 0x11C) & 0x0F) << 8) + (rd_channel((i << 10) + 0x11A) & 0xFF);
				}
				
   	  printf("\n\n");
    	  printf("Channel         :|");
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	        printf("%4X|",i);
    	      }
    	      printf("\n");
    	      printf("                 |");
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	      printf("----|");
    	      }
    	      printf("\n");
    	   
 
    	      printf("VOD             :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
					 printf("%4d|",tx_vodctrl[i]); 	    	  
    	      }
    	      printf("\n");
 
   	      printf("Pre Tap 2 Level :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	           switch (tx_pretap_2[i])
    	           {            
    	           case  PRE_TAP2_7N  : printf("  -7|"); break;
    	           case  PRE_TAP2_6N  : printf("  -6|"); break;
    	           case  PRE_TAP2_5N  : printf("  -5|"); break;
    	           case  PRE_TAP2_4N  : printf("  -4|"); break;
    	           case  PRE_TAP2_3N  : printf("  -3|"); break;
    	           case  PRE_TAP2_2N  : printf("  -2|"); break;
    	           case  PRE_TAP2_1N  : printf("  -1|"); break;
    	           case  PRE_TAP2_0   : printf("   0|"); break;
    	           case  PRE_TAP2_1   : printf("   1|"); break;
    	           case  PRE_TAP2_2   : printf("   2|"); break;
    	           case  PRE_TAP2_3   : printf("   3|"); break;
    	           case  PRE_TAP2_4   : printf("   4|"); break;
    	           case  PRE_TAP2_5   : printf("   5|"); break;
    	           case  PRE_TAP2_6   : printf("   6|"); break;
    	           case  PRE_TAP2_7   : printf("   7|"); break;               
    	           default : break;
    	           } 	    	  
    	      }
   	      printf("\n");
				
    	      printf("Pre Tap 1 Level :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	           switch (tx_pretap_1[i])
    	           {
    	           case  PRE_TAP1_16N : printf(" -16|"); break;						  
    	           case  PRE_TAP1_15N : printf(" -15|"); break;
    	           case  PRE_TAP1_14N : printf(" -14|"); break;
    	           case  PRE_TAP1_13N : printf(" -13|"); break;
    	           case  PRE_TAP1_12N : printf(" -12|"); break;
    	           case  PRE_TAP1_11N : printf(" -11|"); break;
    	           case  PRE_TAP1_10N : printf(" -10|"); break;
    	           case  PRE_TAP1_9N  : printf("  -9|"); break;  
    	           case  PRE_TAP1_8N  : printf("  -8|"); break;              
    	           case  PRE_TAP1_7N  : printf("  -7|"); break;
    	           case  PRE_TAP1_6N  : printf("  -6|"); break;
    	           case  PRE_TAP1_5N  : printf("  -5|"); break;
    	           case  PRE_TAP1_4N  : printf("  -4|"); break;
    	           case  PRE_TAP1_3N  : printf("  -3|"); break;
    	           case  PRE_TAP1_2N  : printf("  -2|"); break;
    	           case  PRE_TAP1_1N  : printf("  -1|"); break;
    	           case  PRE_TAP1_0   : printf("   0|"); break;
    	           case  PRE_TAP1_1   : printf("   1|"); break;
    	           case  PRE_TAP1_2   : printf("   2|"); break;
    	           case  PRE_TAP1_3   : printf("   3|"); break;
    	           case  PRE_TAP1_4   : printf("   4|"); break;
    	           case  PRE_TAP1_5   : printf("   5|"); break;
    	           case  PRE_TAP1_6   : printf("   6|"); break;
    	           case  PRE_TAP1_7   : printf("   7|"); break; 
    	           case  PRE_TAP1_8   : printf("   8|"); break;
    	           case  PRE_TAP1_9   : printf("   9|"); break;
    	           case  PRE_TAP1_10  : printf("  10|"); break;
    	           case  PRE_TAP1_11  : printf("  11|"); break;
    	           case  PRE_TAP1_12  : printf("  12|"); break;
    	           case  PRE_TAP1_13  : printf("  13|"); break;
    	           case  PRE_TAP1_14  : printf("  14|"); break;
    	           case  PRE_TAP1_15  : printf("  15|"); break; 
					  case  PRE_TAP1_16  : printf("  16|"); break; 						  
    	           default : break;
    	           } 	    	  
    	      }
    	      printf("\n");
 
    	      printf("Post Tap 1 Level:|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
    	           switch (tx_posttap_1[i])
    	           {
    	           case  POST_TAP1_0  : printf("   0|"); break;
    	           case  POST_TAP1_1N : printf("  -1|"); break;    
    	           case  POST_TAP1_2N : printf("  -2|"); break;
    	           case  POST_TAP1_3N : printf("  -3|"); break;   
    	           case  POST_TAP1_4N : printf("  -4|"); break;
    	           case  POST_TAP1_5N : printf("  -5|"); break;   
    	           case  POST_TAP1_6N : printf("  -6|"); break;
    	           case  POST_TAP1_7N : printf("  -7|"); break;   
    	           case  POST_TAP1_8N : printf("  -8|"); break;
    	           case  POST_TAP1_9N : printf("  -9|"); break;   
    	           case  POST_TAP1_10N : printf(" -10|"); break;
    	           case  POST_TAP1_11N : printf(" -11|"); break;   
    	           case  POST_TAP1_12N : printf(" -12|"); break;     
    	           case  POST_TAP1_13N : printf(" -13|"); break;   
    	           case  POST_TAP1_14N : printf(" -14|"); break;
    	           case  POST_TAP1_15N : printf(" -15|"); break;   
    	           case  POST_TAP1_16N : printf(" -16|"); break;
    	           case  POST_TAP1_17N : printf(" -17|"); break;   
    	           case  POST_TAP1_18N : printf(" -18|"); break;
    	           case  POST_TAP1_19N : printf(" -19|"); break;   
    	           case  POST_TAP1_20N : printf(" -20|"); break;
    	           case  POST_TAP1_21N : printf(" -21|"); break;   
    	           case  POST_TAP1_22N : printf(" -22|"); break; 
    	           case  POST_TAP1_23N : printf(" -23|"); break;   
    	           case  POST_TAP1_24N : printf(" -24|"); break;
    	           case  POST_TAP1_25N : printf(" -25|"); break;  							  
    	           case  POST_TAP1_1  : printf("   1|"); break;    
    	           case  POST_TAP1_2  : printf("   2|"); break;
    	           case  POST_TAP1_3  : printf("   3|"); break;   
    	           case  POST_TAP1_4  : printf("   4|"); break;
    	           case  POST_TAP1_5  : printf("   5|"); break;   
    	           case  POST_TAP1_6  : printf("   6|"); break;
    	           case  POST_TAP1_7  : printf("   7|"); break;   
    	           case  POST_TAP1_8  : printf("   8|"); break;
    	           case  POST_TAP1_9  : printf("   9|"); break;   
    	           case  POST_TAP1_10 : printf("  10|"); break;
    	           case  POST_TAP1_11 : printf("  11|"); break;   
    	           case  POST_TAP1_12 : printf("  12|"); break;     
    	           case  POST_TAP1_13 : printf("  13|"); break;   
    	           case  POST_TAP1_14 : printf("  14|"); break;
    	           case  POST_TAP1_15 : printf("  15|"); break;   
    	           case  POST_TAP1_16 : printf("  16|"); break;
    	           case  POST_TAP1_17 : printf("  17|"); break;   
    	           case  POST_TAP1_18 : printf("  18|"); break;
    	           case  POST_TAP1_19 : printf("  19|"); break;   
    	           case  POST_TAP1_20 : printf("  20|"); break;
    	           case  POST_TAP1_21 : printf("  21|"); break;   
    	           case  POST_TAP1_22 : printf("  22|"); break; 
    	           case  POST_TAP1_23 : printf("  23|"); break;   
    	           case  POST_TAP1_24 : printf("  24|"); break;
    	           case  POST_TAP1_25 : printf("  25|"); break; 
					  
           
    	           default : break;
    	           }    	  
    	      }
    	      printf("\n");

    	      printf("Post Tap 2 Level:|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 

    	           switch (tx_posttap_2[i])
    	           {

    	           case  POST_TAP2_12N : printf(" -12|"); break;
    	           case  POST_TAP2_11N : printf(" -11|"); break;
    	           case  POST_TAP2_10N : printf(" -10|"); break;
    	           case  POST_TAP2_9N  : printf("  -9|"); break;  
    	           case  POST_TAP2_8N  : printf("  -8|"); break;              
    	           case  POST_TAP2_7N  : printf("  -7|"); break;
    	           case  POST_TAP2_6N  : printf("  -6|"); break;
    	           case  POST_TAP2_5N  : printf("  -5|"); break;
    	           case  POST_TAP2_4N  : printf("  -4|"); break;
    	           case  POST_TAP2_3N  : printf("  -3|"); break;
    	           case  POST_TAP2_2N  : printf("  -2|"); break;
    	           case  POST_TAP2_1N  : printf("  -1|"); break;
    	           case  POST_TAP2_0   : printf("   0|"); break;
    	           case  POST_TAP2_1   : printf("   1|"); break;
    	           case  POST_TAP2_2   : printf("   2|"); break;
    	           case  POST_TAP2_3   : printf("   3|"); break;
    	           case  POST_TAP2_4   : printf("   4|"); break;
    	           case  POST_TAP2_5   : printf("   5|"); break;
    	           case  POST_TAP2_6   : printf("   6|"); break;
    	           case  POST_TAP2_7   : printf("   7|"); break; 
    	           case  POST_TAP2_8   : printf("   8|"); break;
    	           case  POST_TAP2_9   : printf("   9|"); break;
    	           case  POST_TAP2_10  : printf("  10|"); break;
    	           case  POST_TAP2_11  : printf("  11|"); break;
    	           case  POST_TAP2_12  : printf("  12|"); break;
    	                                                                                                                                                                                                           
    	           default : break;
    	           }   	  
    	      }
    	      printf("\n");    	      
				
				printf("CTLE DC Gain    :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 

    	           switch (rx_eqdcgain[i])
    	           {
    	           case  EQ_GAIN_0    : printf("   0|"); break;
    	           case  EQ_GAIN_1    : printf("   1|"); break;
    	           case  EQ_GAIN_2    : printf("   2|"); break;   
    	           case  EQ_GAIN_3    : printf("   3|"); break;
    	           case  EQ_GAIN_4    : printf("   4|"); break;					  
    	           default : break;
    	           } 	  
    	      }
    	      printf("\n");    
				
    	      printf("CTLE AC Gain    :|"); 
    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
    	      { 
					printf("%4d|",rx_eqctrl[i]); 
    	      }
    	      printf("\n"); 

//				printf("DFE Tap 1       :|"); 
//    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    	      { 				
//					tap1 = read_reconfiguration_DFE(i,0x1) &  (0x000F);
//						printf("  %2X|",tap1);
//				}
//				printf("\n");
//
//				printf("DFE Tap 2       :|"); 
//    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    	      { 				
//					tap2 = read_reconfiguration_DFE(i,0x2) &  (0x0007);
//					pol2 =  (read_reconfiguration_DFE(i,0x2)) >> 3  &  (0x0001);
//					if (pol2 == 0 )
//						printf("  %2X|",tap2);
//					else
//						printf(" -%2X|",tap2);
//				}
//				printf("\n");				
//
//				printf("DFE Tap 3       :|"); 
//    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    	      { 				
//					tap3 = read_reconfiguration_DFE(i,0x3) &  (0x0007);
//					pol3 =  (read_reconfiguration_DFE(i,0x3)) >> 3  &  (0x0001);
//					if (pol3 == 0 )
//						printf("  %2X|",tap3);
//					else
//						printf(" -%2X|",tap3);
//				}
//				printf("\n");	
//
//				printf("DFE Tap 4       :|"); 
//    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    	      { 				
//					tap4 = read_reconfiguration_DFE(i,0x4) &  (0x0007);
//					pol4 =  (read_reconfiguration_DFE(i,0x4)) >> 3  &  (0x0001);
//					if (pol4 == 0 )
//						printf("  %2X|",tap4);
//					else
//						printf(" -%2X|",tap4);
//				}
//				printf("\n");	
//
//				printf("DFE Tap 5       :|"); 
//    	      for (i = 0; i < NUMBER_OF_LINKS ; i++)
//    	      { 				
//					tap5 = read_reconfiguration_DFE(i,0x5) &  (0x0007);
//					pol5 =  (read_reconfiguration_DFE(i,0x5)) >> 3  &  (0x0001);
//					if (pol5 == 0 )
//						printf("  %2X|",tap5);
//					else
//						printf(" -%2X|",tap5);
//				}					
//				printf("\n");	
									
    	  break; // Case E

//  		case 'f':
//        case 'F': /*Toggle Reverse Parallel Loopback on channel (if Available) */
//        
//
//  		 if (Rev_Parallel_Available[SelectedChannel] == 0)
//  			 {
//  			 printf("\n\nChannel does not support Reverse Parallel Loopback.\n");
//  			 break;
//  			 }
//  		 else
//			{
//             if (Rev_Parallel[SelectedChannel] == 0)
//          	         Rev_Parallel[SelectedChannel] = 1;
//          	 else
//          	        Rev_Parallel[SelectedChannel] = 0; 
//          	          
//               }
//            Control_Reg = (Control_Reg & (0xFFEF)) | (Rev_Parallel[0] << 4);
//            Control_Reg = (Control_Reg & (0xFFDF)) | (Rev_Parallel[1] << 5);
//            Control_Reg = (Control_Reg & (0xFFBF)) | (Rev_Parallel[2] << 6);
//            Control_Reg = (Control_Reg & (0xFF7F)) | (Rev_Parallel[3] << 7); 
//            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//            
//            Control_Reg = Control_Reg | (0x8000);
//            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//            usleep(10);
//            
//            Control_Reg = Control_Reg & (0x7FFF);
//            IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//            
//            
//                
//          usleep(1000000);
//          
//          printf("Toggled Reverse Parallel Loopback And Reset .\n\n");
//          break;


//      case 'g':
//      case 'G': /* New EyeQ Interval Time Interval */
//        
//        printf("G. Provide EyeQ Time Interval\n");
//        printf("   Insert a Hexadecimal Number from 01 to FF\n");
//        printf("   Current BER Time Interval   : %4d miliseconds\n", EyeQInterval);
//  
//		
//        temp = 0xffffffff;
//           do {
//            printf("\n   New Interval (Higher Nibble) (0-F)        :");
//            rx_char   = input_char();
//        
//            switch (rx_char)
//            {
//            case '0': temp = 0; break;
//            case '1': temp = 1; break;
//            case '2': temp = 2; break;
//            case '3': temp = 3; break;
//            case '4': temp = 4; break;
//            case '5': temp = 5; break;
//            case '6': temp = 6; break;
//            case '7': temp = 7; break;
//            case '8': temp = 8; break;
//            case '9': temp = 9; break;
//            case 'A': 
//            case 'a': temp = 0xa; break;
//            case 'B': 
//            case 'b': temp = 0xb; break;
//            case 'C': 
//            case 'c': temp = 0xc; break;
//            case 'D': 
//            case 'd': temp = 0xd; break;
//            case 'E': 
//            case 'e': temp = 0xe; break;
//            case 'F': 
//            case 'f': temp = 0xf; break;
//            default : break;
//            }
//          } while (temp == 0xffffffff);
//          
//          High_nibble = temp;
//          
//          temp = 0xffffffff;
//           do {
//            printf("\n   New Interval (Lower Nibble) (0-F)        :");
//            rx_char   = input_char();
//        
//            switch (rx_char)
//            {
//            case '0': temp = 0; break;
//            case '1': temp = 1; break;
//            case '2': temp = 2; break;
//            case '3': temp = 3; break;
//            case '4': temp = 4; break;
//            case '5': temp = 5; break;
//            case '6': temp = 6; break;
//            case '7': temp = 7; break;
//            case '8': temp = 8; break;
//            case '9': temp = 9; break;
//            case 'A': 
//            case 'a': temp = 0xa; break;
//            case 'B': 
//            case 'b': temp = 0xb; break;
//            case 'C': 
//            case 'c': temp = 0xc; break;
//            case 'D': 
//            case 'd': temp = 0xd; break;
//            case 'E': 
//            case 'e': temp = 0xe; break;
//            case 'F': 
//            case 'f': temp = 0xf; break;
//            default : break;
//            }
//          } while (temp == 0xffffffff);
//          
//          Low_nibble = temp;  
//    
//          EyeQInterval = (High_nibble << 4) | Low_nibble;
//         
//            
//          printf("\n    New EyeQTime interval is %4d miliseconds:\n",EyeQInterval);               
//            
//       break; /* Case G */
//
//			 
//		case 'm':
//		case 'M':
//		printf("\n");
//		
//		eye1d = 0;	
//		//eyeqbw = 3; //Datarate > 7.5  Gbps 
//		
//		write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0001); // Enable Eye Monitor		
//		
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)
//				printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//		//eyeq_control = (eyeq_control & (0x0000DFFF)) | (eye1d << 13);
//		//eyeq_control = (eyeq_control & (0x0000F9FF)) | (eyeqbw << 9);
//		
//		//write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eye1d to 0 and Eyeq Bandwidth
//		
//		eyepolarity = TOP;
//		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eyepolarity
//		
//		if (DEBUG==1)
//			printf("\neyeq_control = %4x\n",eyeq_control);		
//			
//
//		
//	    ///////////////////////////////////////////////////////////////////////				
//		// Go through all horizontal phase steps and all vertical phase steps one by one and measure the BER
//		///////////////////////////////////////////////////////////////////////	
//		
//		// Upper half first
//            
//		for (x = 0; x < 32 ; x++)
//			{
//				for (y = 0; y < 64 ; y++)
//					{ 
//			
//			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
//			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
//						
//			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
//							
//			usleep(100); // wait for Prbslogic to stabilize
//			
//								   
//			// Reset ErrorCount on Selected Channel
//
//	     Control_Reg = Control_Reg | (0x2000);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//        
//        Control_Reg = Control_Reg & (0xDFFF);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);	
//		
//			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
//	
//			// Wait for programmed time interval
//			do
//			{
//			Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//			} while (Counter_1ms_Reg < TimeInterval);                  
//             
//    
//        
//            //Check if PRBS Lock is valid
//            
//			Channel_Reg[SelectedChannel]     = Read_Channel_Reg(SelectedChannel);
//			
//			Locked[SelectedChannel]  		= 0x0001 & (Channel_Reg[SelectedChannel] >> 15);  
//	
//            //Read ErrorCount registers
//            
//            if (Locked[SelectedChannel] == 1)
//            {			  
//				Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//				ErrorCount_Reg_L[SelectedChannel]     = Read_ErrorCount_L_Reg(SelectedChannel);
//				ErrorCount_Reg_H[SelectedChannel]     = Read_ErrorCount_H_Reg(SelectedChannel);
//				
//						 
//				ErrorCount_Array[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
//	
//				Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);
//	  
//				if (ErrorCount_Array[x][y] > 0)
//					{
//				  BER_Array[x][y] = ((ErrorCount_Array[x][y])) / Totalbits;
//				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
//				  //printf("ErrorCount : %5e\n", ErrorCount_Array[i]);
//				  }
//				else 
//				  BER_Array[x][y] = 0;  
//				  
////				if (ErrorCount_Array[x][y] > 0)
////						printf("Ch %1d Phase Step : %2d BER Estimate : %4e    ErrorCount  : %4e \n",SelectedChannel,i,(BER_Array[i]),ErrorCount_Array[i]);
////					else
////						printf("Ch %1d Phase Step : %2d BER          : 0\n",SelectedChannel,i);			    
//            
//            }
//            else
//			{
//				BER_Array[x][y] = 1;
//				//printf("Ch %1d Phase Step : %2d BER          : 1 (no Prbs Lock)\n",SelectedChannel,i);	
//			}
//			
//																																				
//		} // for loop y
//		
//	} // for loop x
//
//		// Bottom half
//      
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)	
//			printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//	
//		eyepolarity = BOTTOM;
//		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); 
//		if (DEBUG==1)
//			printf("\neyeq_control = %4x\n",eyeq_control);	
//	
//		//write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0001); // Enable Eye Monitor
//	
//		for (x = 0; x < 32 ; x++)
//			{
//				for (y = 1; y < 64 ; y++)
//					{ 
//			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
//			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
//						
//			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
//							
//			usleep(100); // wait for Prbslogic to stabilize
//			
//								   
//			// Reset ErrorCount on Selected Channel
//
//	     Control_Reg = Control_Reg | (0x2000);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//        
//        Control_Reg = Control_Reg & (0xDFFF);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);	
//		
//			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
//	
//			// Wait for programmed time interval
//			do
//			{
//			Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//			} while (Counter_1ms_Reg < TimeInterval);                  
//             
//    
//        
//            //Check if PRBS Lock is valid
//            
//			Channel_Reg[SelectedChannel]     = Read_Channel_Reg(SelectedChannel);
//			
//			Locked[SelectedChannel]  		= 0x0001 & (Channel_Reg[SelectedChannel] >> 15);  
//	
//            //Read ErrorCount registers
//            
//            if (Locked[SelectedChannel] == 1)
//            {			  
//				Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//				ErrorCount_Reg_L[SelectedChannel]     = Read_ErrorCount_L_Reg(SelectedChannel);
//				ErrorCount_Reg_H[SelectedChannel]     = Read_ErrorCount_H_Reg(SelectedChannel);
//				
//						 
//				ErrorCount_Array_Bottom[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
//	
//				Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);
//	  
//				if (ErrorCount_Array_Bottom[x][y] > 0)
//					{
//				  BER_Array_Bottom[x][y] = ((ErrorCount_Array_Bottom[x][y])) / Totalbits;
//				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
//				  //printf("ErrorCount : %5e\n", ErrorCount_Array_Bottom[x][y]);
//				  }
//				else 
//				  BER_Array_Bottom[x][y] = 0;  
//				  
////				if (ErrorCount_Array_Bottom[x][y] > 0)
////						printf("Ch %1d Phase Step : %2d %2d BER Estimate : %4e    ErrorCount  : %4e \n",SelectedChannel,x,y,(BER_Array_Bottom[x][y]),ErrorCount_Array_Bottom[x][y]);
////					else
////						printf("Ch %1d Phase Step : %2d %2d BER          : 0\n",SelectedChannel,x,y);			    
//            
//            }
//            else
//			{
//				BER_Array_Bottom[x][y] = 1;
//				//printf("Ch %1d Phase Step : %2d %2d BER          : 1 (no Prbs Lock)\n",SelectedChannel,x,y);	
//			}
//			
//																																				
//		} // for loop y
//		
//	} // for loop x
//	
//		//printf("\n BATHTUB CURVE ESTIMATE ACROSS 64 PHASES\n");
//	    printf("\n 2-D EyeMargin ESTIMATE ACROSS 32 PHASES (one UI)\n");
//       
//	       ///////////////////////////////////////////////////////////////////////				
//		   // Determine eye opening
//           ///////////////////////////////////////////////////////////////////////				
//	
//		eye_phase = 0;
//		BER_Minimum = 1;
//		BER_Minimum_phase = 0;
//		 
//        for (i = 0; i <32 ;i++)
//        {
//			if (BER_Array[i][0] == 0)
//               eye_phase = eye_phase + 1;
//            if (BER_Array[i][0] < BER_Minimum )
//				{
//				BER_Minimum = BER_Array[i][0];
//				BER_Minimum_phase = i;
//				}
//        }
//        
//			
//
//         eye = (float) eye_phase/32;
//        
//        start_value = 0;
//        end_value = 0;
//        
//        for (i = 1; i <32 ;i++)
//        {
//          if (BER_Array[i][0] != BER_Array[i-1][0])
//          {
//               if (BER_Array[i][0] == 0)
//					start_value = i;
//               else if (BER_Array[i-1][0] == 0)
//                     end_value = i-1;
//           }
//        }
//        //printf("\nstart_value : %2d",start_value);
//        //printf("\nend_value   : %2d",end_value);
//        if (start_value > end_value )
//			end_value = end_value + 32;
//		
//
//		//printf("\nend_value   : %2d",end_value);	
//		if (eye_phase == 0)
//			optimum_phase = BER_Minimum_phase ;
//		else	
//			optimum_phase = start_value + eye_phase/2;
//			
//		//printf("\noptimum phase   : %2d",optimum_phase );
//		if (optimum_phase >= 32 )
//			{
//			table_phase_start = optimum_phase - 15;
//			optimum_phase = optimum_phase -32;
//			}
//		else
//			{
//			table_phase_start = optimum_phase -15;
//			if (table_phase_start < 0)
//				table_phase_start = table_phase_start + 32;
//			}
//		//printf("\noptimum phase   : %2d",optimum_phase );
//		//printf("\ntable_phase_start   : %2d",table_phase_start );	
//
//	   ///////////////////////////////////////////////////////////////////////				
//	   // Find Vertical Eye Opening using optimum horizontal phase
//	   ///////////////////////////////////////////////////////////////////////					
//				
//		veye = 0;
//		 
//        for (i = 0; i <64 ;i++)
//        {
//			if (BER_Array[optimum_phase][i] == 0)
//               veye = veye + 1;
//        }	
//        for (i = 0; i <64 ;i++)
//        {
//			if (BER_Array_Bottom[optimum_phase][i] == 0)
//               veye = veye + 1;
//        }		  
//		
//		veye_top = veye/2 + 10;
//		veye_bottom = veye/2 + 10;  
//		
//			
//	   ///////////////////////////////////////////////////////////////////////				
//	   // Print Centered Eye
//	   ///////////////////////////////////////////////////////////////////////				
//       printf("\nPhase   ");
//       for (i =0;i <32; i++)
//			{
//			index = table_phase_start + i;
//			if (index > 31)
//				index = index -32;
//			if (index < 10) 
//				printf("  ");
//			else if ((index >= 10) && (index < 20))
//			    printf(" 1");
//			else if ((index >= 20) && (index < 30))
//				printf(" 2");
//			else
//				printf(" 3");
//			}
//	  printf("\nStep    ");
//       for (i =0;i <32; i++)
//			{
//			index = table_phase_start + i;
//			if (index > 31)
//				index = index -32;
//			if (index < 10) 
//				printf("%2d",index);
//			else if ((index >= 10) && (index < 20))
//			    printf("%2d",index-10);
//			else if ((index >= 20) && (index < 30))
//				printf("%2d",index-20);
//			else
//				printf("%2d",index-30);
//			}			
//	   printf("\n========================================================================\n");	   	   
//	   for (j = veye_top; j >= 0 ; j--)
//			{   
//			printf("\n%2d     :",j);  
//				for (i = 0; i < 32 ; i++)
//				{ 
//					index = table_phase_start + i;
//					if (index > 31)
//						index = index -32;				    
//					if (BER_Array[index][j] == 0)
//						printf(" .");
//					else
//						printf(" x");
//				}
//			}
//	   for (j = 1; j < veye_bottom ; j++)
//			{   
//			printf("\n-%2d    :",j);  
//				for (i = 0; i < 32 ; i++)
//				{ 
//					index = table_phase_start + i;
//					if (index > 31)
//						index = index -32;				    
//					if (BER_Array_Bottom[index][j] == 0)
//						printf(" .");
//					else
//						printf(" x");
//				}
//			}		
//		printf("\n");				
//                
//         printf("\nHorizontal Eye Opening Ch %1d: %2d phases",SelectedChannel,eye_phase);
//         printf("\nHorizontal Eye Opening Ch %1d: %.2f UI",SelectedChannel,eye);
//         printf("\nOptimum Phase Ch %1d         : %2d",SelectedChannel,optimum_phase);
//         printf("\nVertical Eye Opening Ch %1d  : %2d steps",SelectedChannel,veye);	
//			printf("\nSampling time per sample     : %2d ms",EyeQInterval);				
//			
//			
//	// Set Eyepolarity to TOP again
//	
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)			
//			printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//			
//	eyepolarity = TOP;
//	eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//	write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control);
//			
//   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0000); // Disable Eye Monitor			
//		
//		
//		break;


//		case 'n':
//		case 'N':
//		printf("\n");
//		
//		eye1d = 0;	
//		eyeqbw = 3; //Datarate > 7.5  Gbps 
//		
//		write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0007); // Enable Eye Monitor	and BERB Block	
//		
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)
//				printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//		//eyeq_control = (eyeq_control & (0x0000DFFF)) | (eye1d << 13);
//		//eyeq_control = (eyeq_control & (0x0000F9FF)) | (eyeqbw << 9);
//		
//		//write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eye1d to 0 and Eyeq Bandwidth
//		
//		eyepolarity = TOP;
//		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); // Set eyepolarity
//		
//		if (DEBUG==1)
//			printf("\neyeq_control = %4x\n",eyeq_control);		
//			
//
//		
//	    ///////////////////////////////////////////////////////////////////////				
//		// Go through all horizontal phase steps and all vertical phase steps one by one and measure the BER
//		///////////////////////////////////////////////////////////////////////	
//		
//		// Upper half first
//            
//		for (x = 0; x < 32 ; x++)
//			{
//				for (y = 0; y < 64 ; y++)
//					{ 
//			
//			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
//			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
//						
//			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
//							
//			usleep(100); // wait for Prbslogic to stabilize
//			
//		   
//		   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x000F); // Reset counters
//					   
//
//			// Reset Counter_1MS register
//
//	     Control_Reg = Control_Reg | (0x2000);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//        
//        Control_Reg = Control_Reg & (0xDFFF);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);	
//						
//			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
//	
//			// Wait for programmed time interval
//			do
//			{
//			Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//			} while (Counter_1ms_Reg < TimeInterval);                  
//             
//			
//         write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0017); // Take a snapshot
//			
//			
//  
//				ErrorCount_Reg_L[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x7);
//				ErrorCount_Reg_H[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x8);
//	
//				BitCount_L[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x5);
//				BitCount_H[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x6);			
//				
//			
//				ErrorCount_Array[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
//	
//			   Totalbits = ((float) (BitCount_H[SelectedChannel] ) *  MULTIPLIER * MULTIPLIER) + (float) (BitCount_L[SelectedChannel] );
//				
//				//Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);
//	  
//				if (ErrorCount_Array[x][y] > 0)
//					{
//				  BER_Array[x][y] = ((ErrorCount_Array[x][y])) / Totalbits;
//				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
//				  //printf("ErrorCount : %5e\n", ErrorCount_Array[i]);
//				  }
//				else 
//				  BER_Array[x][y] = 0;  
//				  
//			
//																																				
//		} // for loop y
//		
//	} // for loop x
//
//		// Bottom half
//      
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)	
//			printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//	
//		eyepolarity = BOTTOM;
//		eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//		write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control); 
//		if (DEBUG==1)
//			printf("\neyeq_control = %4x\n",eyeq_control);	
//	
//		//write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0001); // Enable Eye Monitor
//	
//		for (x = 0; x < 32 ; x++)
//			{
//				for (y = 1; y < 64 ; y++)
//					{ 
//			write_reconfiguration_EyeQ(SelectedChannel,0x2,y); // Select vertical phase
//			usleep(10); // Wait for 10 us to stabilize the vertical level to settle down
//						
//			write_reconfiguration_EyeQ(SelectedChannel,0x1,x); // Select horizontal phase
//							
//			usleep(100); // wait for Logic to stabilize
//			
//		   
//		   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x000F); // Reset counters
//
//			// Reset Counter_1MS register
//
//	     Control_Reg = Control_Reg | (0x2000);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//        
//        Control_Reg = Control_Reg & (0xDFFF);
//        IOWR_ALTERA_AVALON_PIO_DATA(CONTROL_REG_BASE,Control_Reg);
//					   
//		
//			TimeInterval = EyeQInterval; // EyeQInterval is in ms.
//	
//			// Wait for programmed time interval
//			do
//			{
//			Counter_1ms_Reg     = IORD_ALTERA_AVALON_PIO_DATA(COUNTER_1MS_REG_BASE);
//			} while (Counter_1ms_Reg < TimeInterval);                  
//             
//			
//         write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0017); // Take a snapshot
//			
//			
//  
//				ErrorCount_Reg_L[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x7);
//				ErrorCount_Reg_H[SelectedChannel]     = read_reconfiguration_EyeQ(SelectedChannel,0x8);
//	
//				BitCount_L[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x5);
//				BitCount_H[SelectedChannel]     		  = read_reconfiguration_EyeQ(SelectedChannel,0x6);			
//				
//			
//				ErrorCount_Array_Bottom[x][y] = ((float) (ErrorCount_Reg_H[SelectedChannel]) *  MULTIPLIER * MULTIPLIER) + (float) (ErrorCount_Reg_L[SelectedChannel]);
//	
//			   Totalbits = ((float) (BitCount_H[SelectedChannel] ) *  MULTIPLIER * MULTIPLIER) + (float) (BitCount_L[SelectedChannel] );
//				
//				//Totalbits = (float) (Counter_1ms_Reg) * (float) (Bitrate*1000);
//
//	  
//				if (ErrorCount_Array_Bottom[x][y] > 0)
//					{
//				  BER_Array_Bottom[x][y] = ((ErrorCount_Array_Bottom[x][y])) / Totalbits;
//				  //printf("\nErrorCount %6d %6d",ErrorCount_Reg_H[SelectedChannel],ErrorCount_Reg_L[SelectedChannel]);
//				  //printf("ErrorCount : %5e\n", ErrorCount_Array_Bottom[x][y]);
//				  }
//				else 
//				  BER_Array_Bottom[x][y] = 0;  
//				  
////				if (ErrorCount_Array_Bottom[x][y] > 0)
////						printf("Ch %1d Phase Step : %2d %2d BER Estimate : %4e    ErrorCount  : %4e \n",SelectedChannel,x,y,(BER_Array_Bottom[x][y]),ErrorCount_Array_Bottom[x][y]);
////					else
////						printf("Ch %1d Phase Step : %2d %2d BER          : 0\n",SelectedChannel,x,y);			    
//            
//			
//																																				
//		} // for loop y
//		
//	} // for loop x
//	
//		//printf("\n BATHTUB CURVE ESTIMATE ACROSS 64 PHASES\n");
//	    printf("\n 2-D EyeMargin ESTIMATE ACROSS 32 PHASES (one UI) using BERB\n");
//       
//	       ///////////////////////////////////////////////////////////////////////				
//		   // Determine eye opening
//           ///////////////////////////////////////////////////////////////////////				
//	
//		eye_phase = 0;
//		BER_Minimum = 1;
//		BER_Minimum_phase = 0;
//		 
//        for (i = 0; i <32 ;i++)
//        {
//			if (BER_Array[i][0] == 0)
//               eye_phase = eye_phase + 1;
//            if (BER_Array[i][0] < BER_Minimum )
//				{
//				BER_Minimum = BER_Array[i][0];
//				BER_Minimum_phase = i;
//				}
//        }
//        
//			
//
//         eye = (float) eye_phase/32;
//        
//        start_value = 0;
//        end_value = 0;
//        
//        for (i = 1; i <32 ;i++)
//        {
//          if (BER_Array[i][0] != BER_Array[i-1][0])
//          {
//               if (BER_Array[i][0] == 0)
//					start_value = i;
//               else if (BER_Array[i-1][0] == 0)
//                     end_value = i-1;
//           }
//        }
//        //printf("\nstart_value : %2d",start_value);
//        //printf("\nend_value   : %2d",end_value);
//        if (start_value > end_value )
//			end_value = end_value + 32;
//		
//
//		//printf("\nend_value   : %2d",end_value);	
//		if (eye_phase == 0)
//			optimum_phase = BER_Minimum_phase ;
//		else	
//			optimum_phase = start_value + eye_phase/2;
//			
//		//printf("\noptimum phase   : %2d",optimum_phase );
//		if (optimum_phase >= 32 )
//			{
//			table_phase_start = optimum_phase - 15;
//			optimum_phase = optimum_phase -32;
//			}
//		else
//			{
//			table_phase_start = optimum_phase -15;
//			if (table_phase_start < 0)
//				table_phase_start = table_phase_start + 32;
//			}
//		//printf("\noptimum phase   : %2d",optimum_phase );
//		//printf("\ntable_phase_start   : %2d",table_phase_start );	
//
//	   ///////////////////////////////////////////////////////////////////////				
//	   // Find Vertical Eye Opening using optimum horizontal phase
//	   ///////////////////////////////////////////////////////////////////////					
//				
//		veye = 0;
//		 
//        for (i = 0; i <64 ;i++)
//        {
//			if (BER_Array[optimum_phase][i] == 0)
//               veye = veye + 1;
//        }	
//        for (i = 0; i <64 ;i++)
//        {
//			if (BER_Array_Bottom[optimum_phase][i] == 0)
//               veye = veye + 1;
//        }		  
//		
//		veye_top = veye/2 + 10;
//		veye_bottom = veye/2 + 10;  
//		
//			
//	   ///////////////////////////////////////////////////////////////////////				
//	   // Print Centered Eye
//	   ///////////////////////////////////////////////////////////////////////				
//       printf("\nPhase   ");
//       for (i =0;i <32; i++)
//			{
//			index = table_phase_start + i;
//			if (index > 31)
//				index = index -32;
//			if (index < 10) 
//				printf("  ");
//			else if ((index >= 10) && (index < 20))
//			    printf(" 1");
//			else if ((index >= 20) && (index < 30))
//				printf(" 2");
//			else
//				printf(" 3");
//			}
//	  printf("\nStep    ");
//       for (i =0;i <32; i++)
//			{
//			index = table_phase_start + i;
//			if (index > 31)
//				index = index -32;
//			if (index < 10) 
//				printf("%2d",index);
//			else if ((index >= 10) && (index < 20))
//			    printf("%2d",index-10);
//			else if ((index >= 20) && (index < 30))
//				printf("%2d",index-20);
//			else
//				printf("%2d",index-30);
//			}			
//	   printf("\n========================================================================\n");	   	   
//	   for (j = veye_top; j >= 0 ; j--)
//			{   
//			printf("\n%2d     :",j);  
//				for (i = 0; i < 32 ; i++)
//				{ 
//					index = table_phase_start + i;
//					if (index > 31)
//						index = index -32;				    
//					if (BER_Array[index][j] == 0)
//						printf(" .");
//					else
//						printf(" x");
//				}
//			}
//	   for (j = 1; j < veye_bottom ; j++)
//			{   
//			printf("\n-%2d    :",j);  
//				for (i = 0; i < 32 ; i++)
//				{ 
//					index = table_phase_start + i;
//					if (index > 31)
//						index = index -32;				    
//					if (BER_Array_Bottom[index][j] == 0)
//						printf(" .");
//					else
//						printf(" x");
//				}
//			}		
//		printf("\n");				
//                
//         printf("\nHorizontal Eye Opening Ch %1d: %2d phases",SelectedChannel,eye_phase);
//         printf("\nHorizontal Eye Opening Ch %1d: %.2f UI",SelectedChannel,eye);
//         printf("\nOptimum Phase Ch %1d         : %2d",SelectedChannel,optimum_phase);
//         printf("\nVertical Eye Opening Ch %1d  : %2d steps",SelectedChannel,veye);	
//			printf("\nSampling time per sample     : %2d ms",EyeQInterval);	
//			
//			
//	// Set Eyepolarity to TOP again
//	
//		eyeq_status = read_reconfiguration_EyeQ(SelectedChannel,0x3);
//		if (DEBUG==1)			
//			printf("\nEyeQstatus = %4x\n",eyeq_status);
//		eyeq_control = eyeq_status;
//			
//	eyepolarity = TOP;
//	eyeq_control = (eyeq_control & (0x0000FFFB)) | (eyepolarity << 2);	
//	write_reconfiguration_EyeQ(SelectedChannel,0x3,eyeq_control);
//			
//   write_reconfiguration_EyeQ(SelectedChannel,0x0,0x0000); // Disable Eye Monitor			
//		
//		
//		break;

		
		
//		case 'q':
//		case 'Q':
//		
//		//write_reconfiguration_DFE(SelectedChannel,0x0,0x0003); // Use this for continuous adaptation
//		write_reconfiguration_DFE(SelectedChannel,0xB,0x0001); // Initiate DFE 1-time adaptation on Selectedchannel.
//		tap1 = read_reconfiguration_DFE(SelectedChannel,0x1) &  (0x000F);
//		tap2 = read_reconfiguration_DFE(SelectedChannel,0x2) &  (0x0007);
//		pol2 = (read_reconfiguration_DFE(SelectedChannel,0x2)) >> 3  &  (0x0001);
//		tap3 = read_reconfiguration_DFE(SelectedChannel,0x3) &  (0x0007);
//		pol3 = (read_reconfiguration_DFE(SelectedChannel,0x3)) >> 3  &  (0x0001);
//		tap4 = read_reconfiguration_DFE(SelectedChannel,0x4) &  (0x0007);
//		pol4 = (read_reconfiguration_DFE(SelectedChannel,0x4)) >> 3  &  (0x0001);
//		tap5 = read_reconfiguration_DFE(SelectedChannel,0x5) &  (0x0007);
//		pol5 = (read_reconfiguration_DFE(SelectedChannel,0x5)) >> 3  &  (0x0001);
//		
//		printf("\n One time DFE run on SelectedChannel");
//		printf("\n Tap 1 : %2X",tap1);
//		if (pol2 ==0 )
//			printf("\n Tap 2 : %2X",tap2);
//		else
//			printf("\n Tap 2 :-%2X",tap2);
//
//		if (pol3 ==0 )
//			printf("\n Tap 3 : %2X",tap3);
//		else
//			printf("\n Tap 3 :-%2X",tap3);
//		
//		if (pol4 ==0 )
//			printf("\n Tap 4 : %2X",tap4);
//		else
//			printf("\n Tap 4 :-%2X",tap4);
//
//		if (pol5 ==0 )
//			printf("\n Tap 5 : %2X",tap5);
//		else
//			printf("\n Tap 5 :-%2X",tap5);
//					
//		 break;	
	

		  

		
//		case 'y':
//		case 'Y':
//		for (j = 0; j < 16 ; j++)
//			{			
//			value[j] = 0;
//			}
//		
//		for (j = 1; j < 10000 ; j++)
//			{ 
//				write_reconfiguration_DFE(SelectedChannel,0xB,0x0001); // Initiate DFE 1-time adaptation on Selectedchannel.	
//				tap1 = read_reconfiguration_DFE(SelectedChannel,0x1) &  (0x000F);
//				switch (tap1)
//				{
//					case 0 : value[0] = value[0] + 1;break;
//					case 1 : value[1] = value[1] + 1;break;
//					case 2 : value[2] = value[2] + 1;break;
//					case 3 : value[3] = value[3] + 1;break;
//					case 4 : value[4] = value[4] + 1;break;
//					case 5 : value[5] = value[5] + 1;break;
//					case 6 : value[6] = value[6] + 1;break;
//					case 7 : value[7] = value[7] + 1;break;
//					case 8 : value[8] = value[8] + 1;break;
//					case 9 : value[9] = value[9] + 1;break;
//					case 10 : value[10] = value[10] + 1;break;
//               default : break;							
//				}
//			}
//				for (j = 0; j < 16 ; j++)
//					{			
//					printf("\nTap 1 Value %2d : %5d", j, value[j]);
//					}
//
//		break;
						
			

	 case 'z': //Dump register space
	 case 'Z':
		for (j = 0; j < NUMBER_OF_LINKS; j++)
		{		
		for (i = 0x134; i <= 0x139 ; i++)
			{   
					//offset = (j<<10)+i;
					temp = rd_channel((j<<10)+i); // From bit 10 it specifies the channel.
					printf("\nChannel %1d Register 0x%3x : 0x%2x",j,i,temp);
			}	
					temp = rd_channel((j<<10)+7); 
					printf("\nChannel %1d Register 0x7 : 0x%2x",j,temp);	

					temp = rd_channel((j<<10)+0xA); 
					printf("\nChannel %1d Register 0xA : 0x%2x",j,temp);	

					temp = rd_channel((j<<10)+0x110); //
					printf("\nChannel %1d Register 0x110 : 0x%2x",j,temp);		
			
					temp = rd_channel((j<<10)+0x111); //
					printf("\nChannel %1d Register 0x111 : 0x%2x",j,temp);	
	
					temp = rd_channel((j<<10)+0x119); //
					printf("\nChannel %1d Register 0x119 : 0x%2x",j,temp);			
		printf("\n");
		}

	   
		for (i = 0x101; i <= 0x117 ; i++)
			{   
					temp = rd_lc(i); 
					printf("\nATX PLL Register 0x%3x : 0x%2x",i,temp);
			}	


			
	 break; 		
	
	 case 'y': //Modify CDR settings
	 case 'Y':
		for (j = 0; j < NUMBER_OF_LINKS; j++)
		{	
		
		//Set dyn to 31 : 0x1F on the position of 0x7C becomes 0x7C;
		rmw_channel( ((j<<10) + 0x137),0x7C,0x7C);
		
		//Set fix to 3 (0x3).
		rmw_channel( ((j<<10) + 0x136),0x0F,0x03);
		rmw_channel( ((j<<10) + 0x134),0x40,0x00);
		
		//set IPD to 3 (0x18)
		rmw_channel( ((j<<10) + 0x139),0x38,0x18);		
		}

	 case 'p': //Recalibrate ATX PLL
	 case 'P':		
		 rmw_lc(0x100,0x01,0x01);
	    rmw_lc(0x000,0x03,0x03); // Correct or not?
		 usleep(100000);
		 temp = rd_lc(0x101);
		 printf("\nATX PLL Register 0x101 : 0x%2x",temp);
//ATX PLL Recalibration
//rmw $m $base_addr 0x100 0x01 0x01
//rmw $m $base_addr 0x000 0x03 0x03
//rd  $m $base_addr 0x101
//Status is address 0x101 bit[0] 
//           0x0 = Fail; 0x1=Pass
		
	 break; 			
	

	 case 'q': 
	 case 'Q':		
		for (j = 0; j < NUMBER_OF_LINKS; j++)
		{	
		
		//Select serialization factor of 64 	
      rmw_channel( ((j<<10) + 0x110),0x07,0x03);			
		//Select Local CGB for working with TTK
		rmw_channel( ((j<<10) + 0x111),0xFF,0x98);

		}
		
	 break; 
	 
	 case 'r': //Recalibrate
	 case 'R':
			IOWR(RECONFIG_MGMT_0_BASE,0x0,0x0);	
					printf("\nRequest recalibration");	
	 break; 	
					  

	 case 's': //Dump register space of SelectedChannel
	 case 'S':	
		for (i = 0x0; i <= 0x256 ; i++)
			{   
					//offset = (j<<10)+i;
					dump_register_before[i] = rd_channel((SelectedChannel <<10)+i); 
			}	
			printf("\n\nStored all registers from channel %2d in memory\n\n",SelectedChannel);
			
	 break; 
		
	 case 't': //Dump register space of channel 0 after and compare
	 case 'T':	
		for (i = 0x0; i <= 0x256 ; i++)
			{   
					dump_register_after[i] = rd_channel((SelectedChannel <<10)+i); ; 
				   if (dump_register_after[i] != dump_register_before[i])
						printf("\nChannel %2d : Register 0x%3x : Before :0x%2x  After : 0x%2x",SelectedChannel,i,dump_register_before[i],dump_register_after[i]);
			}	
			printf("\n\n");
	 break; 			
			
     case '0':
        printf("\nTerminating Program.\n");
        return 0;
       break;
  
      default:
        printf("\n\tPlease enter a value from 0 to E\n\n\n");
        break;
    }
 }
 return 0;
}
